% Main function for running 3DDA
% Written by A. C. Kellerman
% runs VERB in one step mode with constant boundary conditions but assimilates data
% on each of the three diffusion operations.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function da3d(fcopt)

% create simulation directory if necessary

make_sim_dir(fcopt)

fcopt.minpsd = 1e-31; %fixed

%pp = gcp();
pp=1;
fcopt.d_time = fcopt.timestep/24 ; %time step in days

start_time = fcopt.c_date;
end_time = fcopt.c_date + 1;

%% time to check the current grid file

dxx_folder = [fcopt.simulation_dir,'DiffCoeff/'];
fck = dir([dxx_folder,'perp_grid.plt']);
if ~isempty(fck)
    [dxxL, dxxepc, dxxalpha, ~] = load_plt([dxx_folder, 'perp_grid.plt']);

    gc_ok = true;
    if dxxL.arr(1,1,1) ~= fcopt.L_min || ... 
        dxxL.arr(end,1,1) ~= fcopt.L_max || ... 
        size(dxxL.arr,1) ~=  fcopt.NL  
        fprintf('Lstar grid missmatch\n')
        gc_ok = false;
    end

    if dxxepc.arr(end,1,1) ~= fcopt.E_min || ... 
        dxxepc.arr(end,end,1) ~= fcopt.E_max
        fprintf('Energy grid missmatch\n')
        gc_ok = false;
    end

    if abs(dxxalpha.arr(end,1,1).*180/pi - fcopt.a_min) > 1e-4 || ... 
        abs(dxxalpha.arr(end,1,end).*180/pi - fcopt.a_max) > 1e-4 
        fprintf('Pitch Angle grid missmatch\n')
        gc_ok = false;
    end
else
    gc_ok = false;
end 
% if grid is not ok, then we need to recompute:
if (~gc_ok)
    % we need to reinterpolate dxx
    % first step is to run VERB and let it fail
    % it will compute a new grid in the processes
    if fcopt.Dxx_opt > 0
        create_dxx_ini(fcopt.simulation_dir,fcopt.Dxx_opt);
    end
    VERB_dummy(fcopt) %run simulation to get the grid
    if fcopt.Dxx_opt > 0
        Interp_Dxx_old(fcopt.simulation_dir,fcopt.Dxx_opt);
    end
end 

% here we can restart the simulation, if desired
rsflag = false;
if fcopt.restart_simulation
    prev_fc = load(fcopt.restart_file);
    IPSD = squeeze(prev_fc.simulation_PSD(end,:,:,:));
    rsflag = true; % success
end

% the time range of the simulation
time_range = [start_time end_time];

% this file contains the grid data, specified in Ini_dirs.m
folder = [fcopt.simulation_dir,'Output/'];
[L, epc, alpha, pc] = load_plt([folder, 'perp_grid.plt']);
model_lstar = L.arr;
model_energy = epc.arr;
model_alpha = alpha.arr;
model_pc = pc.arr;

lvals = model_lstar(:,end,end);
model_invk = Lalpha2K(model_lstar,model_alpha);
model_invmu = pc2mu(model_lstar,model_pc,model_alpha);


%% specification of initial analysis error covariance matrices
if ~exist('Pa_alpha','var') || fcopt.reset_cov_mat
    Pa_alpha=zeros(fcopt.NL,fcopt.NE,fcopt.NA,fcopt.NA); % then initialize as zero
end
if ~exist('Pa_pc','var') || fcopt.reset_cov_mat;
    Pa_pc=zeros(fcopt.NL,fcopt.NA,fcopt.NE,fcopt.NE);
end
if ~exist('Pa_L','var') || fcopt.reset_cov_mat;
    Pa_L=zeros(fcopt.NE,fcopt.NA,fcopt.NL,fcopt.NL);
end

%% create the simulation time array
simulation_time = time_range(1):fcopt.d_time:time_range(end);
nt = length(simulation_time);

rbtxt=''; %unused

clear spc_time spc_PSD spc_Lstar spc_InvK spc_InvMu 
hptxt = '';  %unused

%sd = start_time;
%if fcopt.prog_plot
%    ed = end_time;
%else
%    ed = fcopt.utc;
%end

%% load spacecraft data:
% specify the possible spc inputs:

 %test_spcs={'rbspa','rbspb','rbspa','rbspb'};
 %test_instr={'mageis','mageis','rept','rept'};

% test_spcs={'rbspa','rbspb','rbspa','rbspb','noaa','noaa_cc'};
% test_instr={'mageis','mageis','rept','rept','SEM2Peck'};

%test_spcs={'noaa_reg'};
% test_spcs={'noaa'};
% test_instr={'SEM2Peck'};

%test_spcs={'lomonosov_cc'};
%test_instr={'EPD'};

% test with empty satellite data
% test_spcs={'1'};
% test_instr={'2'};

 test_spcs={'rbspa','rbspa'};
 test_instr={'mageis','rept'};

test_calib=zeros(1,length(test_spcs)); %unused

%% load the data
ns = length(test_spcs);
nspc=0;
spcs=[];
for ispc=1:ns    
    tmp = get_finalpsd_data(test_spcs{ispc},test_instr{ispc},test_calib(ispc),fcopt,[start_time end_time]);
    
    if ~isempty(tmp)    
        eval([test_spcs{ispc},test_instr{ispc},' = tmp;']);
        nspc = nspc + 1;
        spcs{nspc} = [test_spcs{ispc},test_instr{ispc}];
    end 
end

nt2 = length(simulation_time);
nspc = length(spcs);

PSD_obs = nan(nspc,nt2,fcopt.NL,fcopt.NE,fcopt.NA);
PSD_obs_err = nan(nspc,nt2,fcopt.NL,fcopt.NE,fcopt.NA);

%warning('off','MATLAB:scatteredInterpolant:InterpEmptyTri2DWarnId');
%warning('off','MATLAB:scatteredInterpolant:DupPtsAvValuesWarnId');
%warning('off','MATLAB:scatteredInterpolant:TooFewPtsInterpWarnId');

%interpolate PSD, and populate PSD_obs array, remove arrays from memory as we go
for ispc=1:nspc
    fprintf(['interpolating ',spcs{ispc},' PSD...']);
    eval(['tmp = ',spcs{ispc},';']);
    eval(['clear ',spcs{ispc}]);
    [tmp_g] = interp2grid(simulation_time,model_invmu,model_invk,model_lstar,tmp.Time,tmp.InvMu,tmp.InvK,tmp.Lstar,tmp.PSD);
    %[tmp_g] = interp2grid2(simulation_time,model_invmu,model_invk,model_lstar,tmp.Time,tmp.InvMu,tmp.InvK,tmp.Lstar,tmp.PSD);
    PSD_obs(ispc,:,:,:,:) = tmp_g .* 2.997e7 ; %convert to VERB units

    gridsize = length(tmp_g(:));
    validpts = find(~isnan(tmp_g));

    clear tmp_g 			
end

%warning('on','MATLAB:scatteredInterpolant:InterpEmptyTri2DWarnId');
%warning('on','MATLAB:scatteredInterpolant:DupPtsAvValuesWarnId');
%warning('on','MATLAB:scatteredInterpolant:TooFewPtsInterpWarnId');

%% obs errors
% errors = sum ( w.*PSD) ./ (w)

% Change to the built-in MATLAB mean function provide predictable return. 
% Absend data is converted to NaN and the function keeps the coorect dimensions. 
PSD_obs = squeeze(mean(PSD_obs,1,'omitnan'));

% % if for some reason we do not have any data, this creates a observation array which is of size(nt,NL,NE,NA), with all NaN entries.
if ndims(PSD_obs) == 3
    PSD_obs = reshape(PSD_obs,[1 size(PSD_obs)]);
    PSD_obs = repmat(PSD_obs,nt,1);
end
% PSD_mod_err = ones(fcopt.NL,fcopt.NE,fcopt.NA) .* fcopt.mod_err_fix;

%% initialize simulation PSD array
simulation_PSD = nan(nt,fcopt.NL,fcopt.NE,fcopt.NA);

%% get kp and compute Lpp
[Kp_smooth,Kp_fixed] = ready_Kp2(fcopt,simulation_time);
% Kp_st = get_Kp2;

Lpp.time = (time_range(1)-2) : fcopt.d_time : (time_range(end)+1);
[Lpp.Kp,~] = ready_Kp2(fcopt,Lpp.time);
Lpp.arr = calc_Lpp(Lpp.time, Lpp.Kp);

l_idx = Lpp.time >= time_range(1);
Lpp.time = Lpp.time(l_idx);
Lpp.arr = Lpp.arr(l_idx);
Lpp.Kp = Lpp.Kp(l_idx);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
%%%     Start simulation/assimilation 
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

if ~rsflag
    IPSD = squeeze(run_VERB_SS(fcopt,Kp_smooth(1))); %run simulation to get initial PSD
end

if ~rsflag
    simulation_PSD(1,:,:,:) = IPSD; % set the first point to SS solution
else
    simulation_PSD(1,:,:,:) = IPSD; % in VERB units    
end

if fcopt.fast_Dxx_load
    % matrices don't depend on the current state of the PSD
    % the initial condition is all that we need
    fastPSD.arr = IPSD;
    fastPSD.time = 0;
end

lastPSD.time = 0;

fprintf('\n');
fprintf('Beginning Simulation\n\n');
totaltime = tic;
oKp_sim = -1;
nt_sim = nt;
old_mathash = 'dummy';

% Get all MP 
% interpolate 
 if fcopt.mp
        R = get_mp(simulation_time);
    end

for it=2:nt_sim % simulation always starts at point 2 (first computation is from f(t[1]) to f(t[2]);
    loopt = tic;
    Kp_step = Kp_fixed(it); % what was the kp representative of the transition of PSD
                        % to the current time? We will use this in the simulation
    Kp_sim = Kp_smooth(it); % use the smoothed value

    Lpp_sim = Lpp.arr(it);

    yrng = [-8 8];
    l_rng = [model_lstar(1,1,1) model_lstar(end,1,1)];

    % introduce loop-specific variables
    % data
    if fcopt.ndastep > 1
        % need to reconsider this part    
        %PSD_d_loop = squeeze(nanmean2(PSD_obs(it-(fcopt.ndastep-1):it,:,:,:),1));  
        PSD_d_loop = squeeze(PSD_obs(it,:,:,:));  
    else
        PSD_d_loop = squeeze(PSD_obs(it,:,:,:));  
    end

    % reanalysis/simulation
    PSD_r_loop = squeeze(simulation_PSD(it-1,:,:,:));
    
    %include MP
    if fcopt.mp
        %PSD_r_loop = apply_mp_to_psd (simulation_time(it),PSD_r_loop,model_lstar);
        PSD_r_loop = apply_mp_to_psd2(it,PSD_r_loop,model_lstar,R);
    end
    PSD_r_loop(PSD_r_loop <= fcopt.minpsd | isnan(PSD_r_loop)) = fcopt.minpsd;
    
    obs_err_loop = ones(fcopt.NL,fcopt.NE,fcopt.NA) .* fcopt.obs_err_fix;
    mod_err_loop = ones(fcopt.NL,fcopt.NE,fcopt.NA) .* fcopt.mod_err_fix;

    %% run VERB for this time step

    [~,~] = mkdir([fcopt.simulation_dir,'DiffMat/']);
    code_time = tic;
    if fcopt.fast_Dxx_load
        mp_set = false; %unused in this version
        [omat2,old_mathash] = get_matrices(fcopt,fastPSD,Kp_sim,Lpp_sim,mp_set,...
            old_mathash);
        if ~isempty(omat2)
            omat = omat2;
        end
    else
        omat = run_VERB_fc(fcopt,lastPSD,Kp_sim,Lpp_sim,mp_set);
    end
    tstr1 = format_time(toc(code_time));

    %% the initial psd, or that from the previous loop
    lastPSD.arr = PSD_r_loop;

    % fix the lower E boundary to remain constant
    lastPSD.arr(:,1,:) = IPSD(:,1,:);
  
    %% keep track of the timing
    assimt = tic;        
    %%%diffusion for L
    if fcopt.use_Dll
        for j=1:fcopt.NE
            w = warning ('off','all');
            PSD_L=PSD_r_loop(:,j,:);
            P_L = Pa_L(j,:,:,:);
            for k=1:fcopt.NA
                [PSD_L(:,1,k),matmodel]=matsolve(omat.L_matr_A(j,k,:,:),...
                    omat.L_matr_B(j,k,:),omat.L_matr_C(j,k,:),PSD_L(:,1,k));
                if (mod(it,fcopt.ndastep) == 0)
                    [PSD_L(:,1,k),P_L(1,k,:,:)] = KalmanFilter_1Derrs(...
                        squeeze(PSD_L(:,1,k)),...
                        mod_err_loop(:,j,k),matmodel,squeeze(P_L(1,k,:,:)),...
                        squeeze(PSD_d_loop(:,j,k))',obs_err_loop(:,j,k));
                end
            end
            PSD_r_loop(:,j,:) = PSD_L;
            Pa_L(j,:,:,:) = P_L;
        end
        PSD_r_loop(PSD_r_loop <= fcopt.minpsd | isnan(PSD_r_loop)) = fcopt.minpsd;
    end

    %%%diffusion for energy
    if fcopt.use_Dpp
        for k=1:fcopt.NA
            PSD_E = PSD_r_loop(:,:,k);
            P_pc = Pa_pc(:,k,:,:);
            for i=1:fcopt.NL
                [PSD_E(i,:,1),matmodel]=matsolve(omat.pc_matr_A(i,k,:,:),...
                    omat.pc_matr_B(i,k,:),omat.pc_matr_C(i,k,:),PSD_E(i,:,1));
                if (mod(it,fcopt.ndastep)==0)
                    [PSD_E(i,:,1),P_pc(i,1,:,:)] = KalmanFilter_1Derrs(...
                        squeeze(PSD_E(i,:,1)),squeeze(mod_err_loop(i,:,k))',matmodel,squeeze(P_pc(i,1,:,:)),...
                        squeeze(PSD_d_loop(i,:,k))',squeeze(obs_err_loop(i,:,k))');
                end 
            end                           
            PSD_r_loop(:,:,k) = PSD_E;
            Pa_pc(:,k,:,:) = P_pc;
        end

        PSD_r_loop(PSD_r_loop <= fcopt.minpsd | isnan(PSD_r_loop)) = fcopt.minpsd;
    end

    %%%diffusion for alpha
    if fcopt.use_Daa
        for j=1:fcopt.NE
            PSD_A=PSD_r_loop(:,j,:);
            P_al = Pa_alpha(:,j,:,:);

            for i=1:fcopt.NL
                [PSD_A(i,1,:),matmodel]=matsolve(omat.alpha_matr_A(i,j,:,:),...
                    omat.alpha_matr_B(i,j,:),omat.alpha_matr_C(i,j,:),PSD_A(i,1,:));  
                if (mod(it,fcopt.ndastep)==0)
                    [PSD_A(i,1,:),P_al(i,1,:,:)] = KalmanFilter_1Derrs(...
                        squeeze(PSD_A(i,1,:)),squeeze(mod_err_loop(i,j,:)),matmodel,...
                        squeeze(P_al(i,1,:,:)),...
                        squeeze(PSD_d_loop(i,j,:))',squeeze(obs_err_loop(i,j,:)));
                end
            end
            PSD_r_loop(:,j,:) = PSD_A;
            Pa_alpha(:,j,:,:) = P_al;
        end
        PSD_r_loop(PSD_r_loop <= fcopt.minpsd | isnan(PSD_r_loop)) = fcopt.minpsd;
    end

    %% assign the PSD to the time-dependent array
    simulation_PSD(it,:,:,:) = PSD_r_loop;

    fprintf('Time: %s\n', datestr(simulation_time(it),'yyyy-mm-dd, HH:MM'));
    fprintf('Step: %s/%s, Kp: %s\n',num2str(it),num2str(nt_sim),...
        num2str(Kp_sim,3));
    fprintf('VERB/MATRIX TIME: %s\n',tstr1);
    tstr = format_time(toc(assimt));
    fprintf('ASSIMILATION TIME: %s\n',tstr);
    tstr = format_time(toc(loopt));
    fprintf('TOTAL STEP TIME: %s\n',tstr);
    tstr = format_time(toc(totaltime));
    fprintf('TOTAL SIMULATION TIME: %s\n',tstr);
    fprintf('Estimated Completion: %s\n',estimation_time(totaltime,it-1,nt_sim-1));
    fprintf('################\n')
    fprintf(' \n')
end

Kp = Kp_smooth;

[~,~] = mkdir(fcopt.arc_dir);
fprintf(' end of simulation: %s\n',fcopt.output_file)

fcoptin = fcopt; % save the input settings

%% save the output to file
save(fcopt.output_file,'simulation_PSD','simulation_time','Kp','Lpp','Pa_alpha','Pa_pc','Pa_L','fcoptin');
