% Data assimilative forecast code settings file.
% Written by A. C. Kellerman
% this file setup is utilized by several scripts inside the code.

function fcopt = da_settings

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Begin code options - contained in fcopt structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    fcopt.recompute = true; %[false] % if true, all files will be overwritten
    % add restart option
    fcopt.restart_date=datenum(2013,4,1); %will recompute all files after this date
                                              % useful if we have a difficult period and we want to try something else
                                              % we should probably specify a condition here to actualy 'change' the 
                                              % settings, so that we can copy the psd from one simulation to a new one.
                                              % currently, any change to settings below won't work
                                              % a standard hash function for typical forecast settings would be useful

    fcopt.da_start_date = datenum(2013,4,3); %where to start 3DDA
    fcopt.da_end_date = datenum(2013,4,29); %we will stop prior to reaching this date

    fcopt.verb_exe_name = 'VERB_code_DA_ImpDaaZero2'; %with scaling for new coeff

    fcopt.timestep = 1; %code timestep in hours

%update step for reanalysis (includes data in between):
    fcopt.ndastep=1; %1 - every step
        
% number of cpu's to use if you have the parallel comp package in MATLAB
    fcopt.use_parallel = true;
    fcopt.numcpu = 4;

    fcopt.mod_err_fix = 0.8; %fixed model error
    fcopt.obs_err_fix = 0.2; %fixed data error
    %fcopt.obs_err_fix = 2000.0;

% Field model to load data for, and to then apply later for visualization
% parameters for loading spacecraft data
    fcopt.mfmtxt = 'T89';
    fcopt.pm_ver = 'ver4'; %only applies to final data at the moment
    fcopt.n_str = 'n4_4'; % accuracy is 4 in ds and dphi
    %fcopt.id = 'LOMO-1_';%2=-3 3=-2 4=-0
    %fcopt.id = 'POES_MOD';
    fcopt.id = 'POES-1_Dxx3_';
%all neuman
    fcopt.upper_L = 'Neuman'; %Neuman constant/constant zero
    fcopt.mp = true; %MP shadowing
% capability to load diffusion coefficients and interpolate them to the grid we want
    fcopt.fast_Dxx_load = true; %[false], true - will save matrices for each Kp, and load them instead of recomputing
            % the choice of boundary conditions may affect this, so default is false, use with care.

% some options for different diffusion coefficients, add more as required
    fcopt.Dxx_opt = 3;
            % 0 - use Fa from VERB
            % Ksenia version
            % 1 - Typical ch and hiss
            % 2 - Typical ch, Ksenia14 hiss
            % 3 - Ksenia 15/16 chorus and hiss
            % Bin Bin version
            % 4 - Typical ch and hiss Typical

% some VERB BC conditions:
%    fcopt.constL = false; % [false], true - false uses Neumann boundary df/dL = 0, true uses const PSD on upper L boundary except during MP shadowing times
%    fcopt.use_fixed_lpc = true; % [true], false - keep lower energy constant

% which diffusion to include (no mixed terms)
    fcopt.use_Dll = true;
    fcopt.use_Daa = true;
    fcopt.use_Dpp = true;

% grid settings:
    fcopt.L_min = 1;
    fcopt.L_max = 7;

    fcopt.E_min = 0.01;
    fcopt.E_max = 10;

    fcopt.a_min = 0.3;
    fcopt.a_max = 89.7;

    fcopt.NL = 31;
    fcopt.NE = 30;
    fcopt.NA = 29;
