% Written by Adam C. Kellerman, Jan 2018
% questions should be addressed directly to the Author.
%
% Obtain PSD data with time-range [st et] - matlabd
% at invariant values MuVals, Kvals
% MuVals and KVals can be different sizes, and will determine the output size of 'psd'
% psd is size(nt,nL,nMu,nK); The L values are taken from the grid directly. 

function [time, psd, ic] = get_3dda(st, et, MuVals, KVals)

fcopt = da_settings; 

base_rea_dir = [getenv('HOME'),'/projects/rb-da-framework/3DDA/'];

%% fcopt variables can be overwritten here manually if need be:


%% end settings 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% create named variables based on settings
[fcopt] = get_sim_dir(fcopt);

folder = [fcopt.simulation_dir,'Output/'];
try
    [L, epc, alpha, pc] = load_plt([folder, 'perp_grid.plt']);
catch
    error('%s does not exist!',folder);
end

ic.SimL = L.arr;
ic.SimInvEnergy = epc.arr;
ic.SimInvAlpha = alpha.arr;
ic.SimInvK = Lalpha2K(L.arr,alpha.arr);
ic.SimInvMu = pc2mu(L.arr,pc.arr,alpha.arr);

cdate = st;

time = [];
psd = [];
while cdate <= et
    fdate = datestr(cdate,'yyyymmdd');
    yrmo = datestr(cdate,'yyyymm');
    rea_file = [fcopt.simulation_dir,'Archive/',yrmo,'/PSD_3DDA_',fdate,'.mat'];
    fprintf('getting psd for %s, last: %s...',fdate, datestr(et,'yyyymmdd'));
    fck = dir(rea_file);
    if isempty(fck)        
        fprintf('no file\n');
    else
        [rea_time, rea_psd] = get_rea_psd_data(rea_file,ic.SimL,ic.SimInvMu,ic.SimInvK,...
            fcopt.mfmtxt,fcopt.numcpu,fcopt.simulation_dir,MuVals,KVals);
        time = cat(1,time,reshape(rea_time,[],1));
        psd = cat(1,psd,rea_psd);
        fprintf('ok\n');
    end
    cdate = cdate + 1;
end

zz = find(time >= st & time < et);
if ~isempty(zz)
    time = time(zz);
    psd = psd(zz,:,:,:);
end
