% Written by Adam C. Kellerman, Jan 2018
% questions should be addressed directly to the Author.
%
% Obtain PSD data with time-range [st et] - matlabd
% at invariant values EVals, Kvals
% EVals and AVals can be different sizes, and will determine the output size of 'psd'
% psd is size(nt,nL,nMu,nK); The L values are taken from the grid directly. 

function [time, flux, ic] = get_3dda2(st, et, EVals, AVals)

fcopt = da_settings; 

base_rea_dir = [getenv('HOME'),'/projects/rb-da-framework/3DDA/'];

%% fcopt variables can be overwritten here manually if need be:


%% end settings 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% create named variables based on settings
[fcopt] = get_sim_dir(fcopt);
disp(fcopt.simulation_dir);

folder = [fcopt.simulation_dir,'Output/'];
try
    [L, epc, alpha, pc] = load_plt([folder, 'perp_grid.plt']);
catch
    error('%s does not exist!',folder);
end

ic.SimL = L.arr;
ic.SimInvEnergy = epc.arr;
ic.SimInvAlpha = alpha.arr;
ic.SimInvK = Lalpha2K(L.arr,alpha.arr);
ic.SimInvMu = pc2mu(L.arr,pc.arr,alpha.arr);

cdate = st;

time = [];
flux = [];
while cdate <= et
    fdate = datestr(cdate,'yyyymmdd');
    yrmo = datestr(cdate,'yyyymm');
    rea_file = [fcopt.simulation_dir,'Archive/',yrmo,'/PSD_3DDA_',fdate,'.mat'];
    fprintf('getting psd for %s, last: %s...',fdate, datestr(et,'yyyymmdd'));
    fck = dir(rea_file);
    if isempty(fck)        
        fprintf('no file\n');
    else
        [rea_time, rea_flux] = get_rea_flux_data(rea_file,ic.SimL,ic.SimInvEnergy,ic.SimInvAlpha,...
            fcopt.mfmtxt,fcopt.numcpu,fcopt.simulation_dir,EVals,AVals);
        time = cat(1,time,reshape(rea_time,[],1));
        flux = cat(1,flux,rea_flux);
        fprintf('ok\n');
    end
    cdate = cdate + 1;
end

zz = find(time >= st & time < et);
if ~isempty(zz)
    time = time(zz);
    flux = flux(zz,:,:,:);
end
