%% Load and plot Reanalisys PSD with Kp

clear
addpath('../Functions/')

% name = '02_base'; % name of the simluation
%name = '02_max_kp9'; % name of the simluation
name = '01_kp_p_4'; % name of the simluation

t1=datenum(2013,4,20); % start time
t2=datenum(2013,4,30); % end time

MuVals = [700]; % can be any size
KVals = [0.1]; % can be any size

trng = [t1 t2]; % time range
lrng = [1 7]; % L range
krng = [0 9]; % Kp range

tticks = t1:2:t2; % define the tick marks


% loads reanalysis, based on da_settings
[rea_time,rea_psd,rea_ic] = get_3dda(t1,t2,MuVals,KVals);
rpsd = squeeze(rea_psd(:,:,1,1));

%%
figure('Position',[100 100 700 900],'defaultAxesFontSize',12,'defaultAxesFontWeight','bold');
%%

% Clim detection
maxpsd = ceil(log10(max(rpsd(:))))-1;
minpsd = maxpsd - 4;
psd_rng = [minpsd maxpsd];

ax1 = subplot(3,1,1);
Kp = Load_Kp2(t1, t2);
stairs(Kp.kp_time, Kp.kp_index,'k','LineWidth',2);
xlim(trng)
ax1.XTick = tticks;
datetick('x','mmm-dd','keeplimits','keepticks')
title('Kp');
ylim(krng);

ax2 = subplot(3,1,[2:3]);
pcolor(rea_time,rea_ic.SimL(:,1,1),log10(rpsd)')
ax2.XLim = ax1.XLim;
ax2.XTick = ax1.XTick;
ax2.XTickLabel = ax1.XTickLabel;
title(sprintf('PSD, \\mu=%2.1f MeV/G, K=%2.1f G^{1/2}R_E',MuVals(1),KVals(1)));

c1 = colorbar;
caxis(psd_rng)
shading flat
colormap(jet);
ylim(lrng);
ylabel('L*');
c1.Label.String = 'log_{10} PSD, (c/cm/MeV)^3';

drawnow;
ax1.Position(3) = ax2.Position(3);
fprintf('mu = %f, K = %f\n',MuVals(1),KVals(1));

saveas(gcf,['~/simple-da-framework/Results/Test_high_kp/' name '.png']); disp('saved');