function Lpp = calc_Lpp(time, Kp)
    %%
    % CALC_LPP calculates plasmapause location Lpp
    % following Carpenter and Anderson [19xx]
    %
    % Lpp = calc_Lpp(time, Kp)
    %
    % Input:
    %   time in days
    %   Kp index
    %
    % Output:
    %   Lpp - plasmapause location in Re

    % Author: Yuri Shprits
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    
    Lpp = zeros(size(time));
    for it = 1:length(time)
        [~, first_idx] = min(abs(time - (time(it) - 1)));
        Kp24 = max(Kp([first_idx:it]));
        Lpp(it) = 5.6 - 0.46*Kp24;
    end

end
