% get final psd data
% written by A. C. Kellerman

function OD = get_finalpsd_data(spc,instr,calib,fcopt,time_range,varargin);

mfmtxt = fcopt.mfmtxt;
use_goes_final = false; %handy option that will use goes realtime, but default to final if it's available. 
if nargin > 5 
    for it=1:1:nargin-5
        if strcmp(varargin(it), 'goes_final')
            use_goes_final=true;
        elseif strcmp(varargin(it),'mfm')
            mfmtxt=varargin{it+1};
            it = it + 1;
        end
    end 
end

if strcmpi(spc,'rbspa') 
    data_dir = [getenv('FC_DATA_DIR'),'/RBSP/rbspa/'];
	spctxt='rbspa';
	td_ech = true;
elseif strcmpi(spc,'rbspb') 
    data_dir = [getenv('FC_DATA_DIR'),'/RBSP/rbspb/'];
	spctxt='rbspb';
	td_ech = true;
elseif strcmpi(spc,'goes13')
    data_dir = [getenv('FC_DATA_DIR'),'/GOES/GOES13/'];
	spctxt='GOES13';
	td_ech = false;
elseif strcmpi(spc,'goes14')
    data_dir = [getenv('FC_DATA_DIR'),'/GOES/GOES14/'];
	spctxt='GOES14';
	td_ech = false;
elseif strcmpi(spc,'goes15')
    data_dir = [getenv('FC_DATA_DIR'),'/GOES/GOES15/'];
	spctxt='GOES15';
	td_ech = false;
elseif strcmpi(spc,'noaa_cc')
    data_dir = [getenv('FC_DATA_DIR'),'/POES/noaa_cc/'];
	spctxt='noaa_cc';
	td_ech = true;
elseif strcmpi(spc,'noaa')
    data_dir = [getenv('FC_DATA_DIR'),'/POES/noaa/'];
	spctxt='noaa';
	td_ech = false;
elseif strcmpi(spc,'noaa_reg')
    data_dir = [getenv('FC_DATA_DIR'),'/POES/noaa_reg/'];
	spctxt='noaa_reg';
	td_ech = true;
elseif strcmpi(spc,'lomonosov_cc')
    data_dir = [getenv('FC_DATA_DIR'),'/Lomonosov/lomonosov_cc/'];
	spctxt='lomonosov_cc';
	td_ech = true;
else
    data_dir = '';
	fprintf('%s not found, add here\n')
 	warning('enter a valid option next time');
end

c_date = floor(time_range(1));

sdir = [data_dir,'Processed_Mat_Files/'];

fprintf('loading %s %s...',spc,instr);
spc_time = [];
spc_PSD = [];
spc_errPSD = [];
spc_Lstar = [];
spc_InvK = [];
spc_InvMu = [];

rbtxt=''; %unused
try
	while c_date <= time_range(end)
		[yr,mo,~,~,~,~] = datevec(c_date);
	  	dates = make_dates(yr,mo);
       % if calib
	%	    file = [sdir,spctxt,'_',instr,'_',dates,'_psd_',fcopt.n_str,'_',fcopt.mfmtxt,...
    %            '_',fcopt.pm_ver,rbtxt,'_calibrated.mat']; load(file);
    %    else
		    file = [sdir,spctxt,'_',instr,'_',dates,'_psd_',fcopt.pm_ver,'.mat']; load(file);
%        end
		file = [sdir,spctxt,'_',instr,'_',dates,'_lstar_',fcopt.n_str,'_',fcopt.mfmtxt,...
			'_',fcopt.pm_ver,'.mat']; load(file);
		file = [sdir,spctxt,'_',instr,'_',dates,'_invmu_and_invk_',fcopt.n_str,'_',...
			fcopt.mfmtxt,'_',fcopt.pm_ver,'.mat']; load(file);
		file = [sdir,spctxt,'_',instr,'_',dates,'_alpha_and_energy_',fcopt.n_str,'_',...
			fcopt.mfmtxt,'_',fcopt.pm_ver,'.mat']; load(file);
		clear alpha_eq_model
%			file = [sdir,spctxt,'_',instr,'_',dates,'_real_invmu_and_alpha_',fcopt.n_str,'_',...
%				fcopt.mfmtxt,'_',fcopt.pm_ver,'.mat']; load(file);
%			InvMu = InvMu_real;
%			clear alpha_eq_real InvMu_real

		if strcmp(instr,'MAGEDandEPEAD')
			ok_ech=find(energy_channels <= 2);
			PSD = PSD(:,ok_ech,:);
			InvMu = InvMu(:,ok_ech,:);
		elseif strcmp(instr,'mageisNoCorr')
			for itm=1:length(time)
				bad_ech=find(energy_channels(itm,:) > 2);
				if ~isempty(bad_ech)
					PSD(itm,bad_ech,:) = NaN;
					InvMu(itm,bad_ech,:) = NaN;
				end
			end
		elseif strcmp(instr,'mageis')
			for itm=1:length(time)
				bad_ech=find(energy_channels(itm,:) > 2);  
				if ~isempty(bad_ech)
					PSD(itm,bad_ech,:) = NaN;    
					InvMu(itm,bad_ech,:) = NaN;  
				end
			end
		elseif strcmp(instr,'SST_FEDU')
			meanl = nanmean2(Lstar,2);
			bad_times = find(meanl < 5);
			Lstar(bad_times,:) = nan;
			PSD(bad_times,:,:) = nan;
		end

		%remove very low values
		for ie=1:size(PSD,2)
			tpsd = squeeze(PSD(:,ie,:));
			tpsd(tpsd < 10.^(log10(nanmean2(tpsd(:))) - 7)) = NaN;
			PSD(:,ie,:) = tpsd;
		end
 		zz=find(time >= time_range(1) & time < time_range(2));
		time = time(zz);
		PSD = double(PSD(zz,:,:));
		Lstar = Lstar(zz,:);
		InvK = InvK(zz,:);
		InvMu = double(InvMu(zz,:,:));

		spc_time = cat(1,spc_time,time);
		spc_PSD = cat(1,spc_PSD,PSD);
		spc_Lstar = cat(1,spc_Lstar,Lstar);
		spc_InvK = cat(1,spc_InvK,InvK);
		spc_InvMu = cat(1,spc_InvMu,InvMu);
		dim = find_DIM(yr,mo);

	 	c_date = datenum(yr,mo,dim) + 1;
    end
end
if ~isempty(spc_time)
	fprintf('done\n');
    OD.Time = spc_time;
    OD.PSD = spc_PSD;
    OD.errPSD = spc_errPSD;
    OD.Lstar = spc_Lstar;
    OD.InvMu = spc_InvMu;
    OD.InvK = spc_InvK;
else
    OD = [];
    fprintf('no data available for %s | %s ---> %s\n',spc,datestr(time_range(1)),datestr(time_range(2)));
end
