% prepare Kp for simulation use
% written by A. C. Kellerman

function [Kp_smooth,Kp_fixed] = ready_Kp(fcopt,time_in)

    sKp = get_Kp2; %3-hour centered Kp

    d_time = fcopt.timestep/24;

    sKp.kp_index(sKp.kp_index == 0) = 0.01;

    % the original time (time_in) corresponds to the 'simulation time'
    % that is [1] is initial condition, [2] is the simulation result after the first step
    % if we shift the time to the midpoint, then we are obtaining the kp that 'best represents' the interval
    % from 1 --> 2
    % to reference this we'd have to use kp(idx - 1) which is a pain, and may be confusing, so instead,
    % we can interpolate Kp to the simulation time (idx) - dt/2. That way the Kp at idx refers to the step idx-1 to idx

    Kp_t = time_in - d_time/2;

    idx=find(~isnan(sKp.kp_index));
    % Kp smooth is the Kp the best represents the interval [1] --> [2], saved for index [2]
    % to reference it in the da code we should choose Kp_smooth [idx]
    Kp_smooth = interp1(sKp.kp_time(idx),sKp.kp_index(idx),Kp_t,'linear');
    Kp_fixed = interp1(sKp.kp_time(idx),sKp.kp_index(idx),Kp_t,'nearest','extrap');

    % in the VERB code we have n outputs, for n-1 steps. Iteration idx refers to the step idx-1 --> idx
    % and so the kp at idx should be interpolated to sim time -dt/2 
    % in order for the verb 

    idx2 = find(isnan(Kp_smooth));
    if ~isempty(idx2)
        notnan = find(~isnan(Kp_smooth));
        if isempty(notnan)
            error('no valid kp found')
        else
            Kp_smooth(idx2) = interp1(Kp_t(notnan),Kp_smooth(notnan),Kp_t(idx2),'nearest','extrap');
        end
    end

