% prepare Kp for simulation use based on omniwebdata
% written by A. Drozdov

function [Kp_smooth,Kp_fixed] = ready_Kp2(fcopt,time_in)  
    d_time = fcopt.timestep/24;

    % the original time (time_in) corresponds to the 'simulation time'
    % that is [1] is initial condition, [2] is the simulation result after the first step
    % if we shift the time to the midpoint, then we are obtaining the kp that 'best represents' the interval
    % from 1 --> 2
    % to reference this we'd have to use kp(idx - 1) which is a pain, and may be confusing, so instead,
    % we can interpolate Kp to the simulation time (idx) - dt/2. That way the Kp at idx refers to the step idx-1 to idx

    Kp_t = time_in - d_time/2;
    
    % Omniweb data work with the whole daty dates. Round to the nearest boundary
    % We take previous 1.5 hour to later center it to 3h
           
    time_start = floor(min(Kp_t-1.5/24));
    time_end   = ceil(max(Kp_t));
    
    omniKp = Load_Kp2(time_start, time_end); % omni Kp    
       
    % sKp = get_Kp2; %3-hour centered Kp    
    % this line from Adam's code is unclear and it is been commented out.
    % sKp.kp_index(sKp.kp_index == 0) = 0.01;     

    idx=~isnan(omniKp.kp_index);
    
    % 3h centered    
    if numel(omniKp.kp_time(idx)) > 1
        kp_3d.kp_time = (time_start:3/24:time_end)+1.5/24;
        kp_3d.kp_index = interp1(omniKp.kp_time, omniKp.kp_index, kp_3d.kp_time,'nearest','extrap');
        omniKp = kp_3d; 
        idx=~isnan(omniKp.kp_index);
        clear kp_3d        
    end
    
    % Kp smooth is the Kp the best represents the interval [1] --> [2], saved for index [2]
    % to reference it in the da code we should choose Kp_smooth [idx]
    
    % Add check for th interpolation data
    if numel(omniKp.kp_time(idx)) > 1
        Kp_smooth = interp1(omniKp.kp_time(idx),omniKp.kp_index(idx),Kp_t,'linear');
        
        % in the VERB code we have n outputs, for n-1 steps. Iteration idx refers to the step idx-1 --> idx
        % and so the kp at idx should be interpolated to sim time -dt/2 
        % in order for the verb 

        idx2 = isnan(Kp_smooth);
        if ~isempty(idx2)
            notnan = ~isnan(Kp_smooth);
            if isempty(notnan)
                warning('no valid kp found');
            else
                Kp_smooth(idx2) = interp1(Kp_t(notnan),Kp_smooth(notnan),Kp_t(idx2),'nearest','extrap');
            end
        end
        
    elseif numel(omniKp.kp_time(idx)) > 0 % return the same point
        Kp_smooth = omniKp.kp_time(idx);
    else
        Kp_smooth = NaN;
    end
    
    % If we don't have valud data we return nothing;
    if numel(omniKp.kp_time(idx)) > 0
        Kp_fixed = interp1(omniKp.kp_time(idx),omniKp.kp_index(idx),Kp_t,'nearest','extrap');
    else
        Kp_fixed = NaN;
    end
end


