% run VERB for steady state condition
%
% written by A. C. Kellerman

%% parameters structure:
function IPSD = run_VERB_SS(fcopt,Kp_set)

    steps=25;
    fid = fopen([fcopt.simulation_dir,'Parameters.ini'], 'w');
    fprintf(fid, ['outputLvl = 0\n']);
    fprintf(fid, ['timeStep = ', num2str(fcopt.timestep,20), '\n']);
    fprintf(fid, ['nDays = ', num2str(fcopt.d_time*steps,20), '\n']);
    fprintf(fid, 'general_Output_parameters.iterStep = %i\n',steps-1);

    fprintf(fid, 'useKp = constant\n');
    fprintf(fid, ['constKp = ', num2str(Kp_set,20), '\n']);

    fprintf(fid, ['localDiffusionsGrid_L.size = ', num2str(fcopt.NL), '\n']);
    fprintf(fid, ['localDiffusionsGrid_L.min = ', num2str(fcopt.L_min), '\n']);
    fprintf(fid, ['localDiffusionsGrid_L.max = ', num2str(fcopt.L_max), '\n']);

    fprintf(fid, ['localDiffusionsGrid_epc.size = ', num2str(fcopt.NE), '\n']);
    fprintf(fid, ['localDiffusionsGrid_epc.min = ', num2str(fcopt.E_min), '\n']);
    fprintf(fid, ['localDiffusionsGrid_epc.max = ', num2str(fcopt.E_max), '\n']);

    fprintf(fid, ['localDiffusionsGrid_alpha.size = ', num2str(fcopt.NA), '\n']);
    fprintf(fid, ['localDiffusionsGrid_alpha.min = ', num2str(fcopt.a_min), '\n']);
    fprintf(fid, ['localDiffusionsGrid_alpha.max = ', num2str(fcopt.a_max), '\n']);

    fprintf(fid, ['L_LowerBoundaryCondition.type =  BCT_CONSTANT_VALUE\n']);
    fprintf(fid, ['L_LowerBoundaryCondition.value =  0\n']);
    fprintf(fid, ['L_UpperBoundaryCondition.type =  BCT_CONSTANT_PSD\n']);

    fprintf(fid, ['pc_UpperBoundaryCondition.type =  BCT_CONSTANT_VALUE\n']);
    fprintf(fid, ['pc_UpperBoundaryCondition.value =  0\n']);
    fprintf(fid, ['pc_LowerBoundaryCondition.type = BCT_CONSTANT_PSD\n']); 

    fprintf(fid, ['alpha_UpperBoundaryCondition.type = BCT_CONSTANT_DERIVATIVE\n']);
    fprintf(fid, ['alpha_UpperBoundaryCondition.value = 0\n']);
    fprintf(fid, ['alpha_LowerBoundaryCondition.type = BCT_CONSTANT_VALUE\n']);
    fprintf(fid, ['alpha_LowerBoundaryCondition.value = 0\n']);

    fprintf(fid, ['psdRadialDiffusion.initial_PSD_Type = IPSDT_STEADY_STATE\n']);
    fprintf(fid, ['psdRadialDiffusion.initial_PSD_Kp0 = ',num2str(Kp_set,20),'\n']); 

    fprintf(fid, 'useRadialDiffusion = Yes\n');
    fprintf(fid, 'useAlphaDiffusion = Yes\n');
    fprintf(fid, 'useEnergyDiffusion = Yes\n');
    fprintf(fid, 'usePcAlphaMixedTerms = No\n');
    fprintf(fid, ['outputModelMatrix = false\n']);
    fprintf(fid, ['solveMatrix = true\n']);

    fprintf(fid, 'localDiffusionsGrid_alpha.useLogScale = No\n'); %default
    fclose(fid);

    %system(['cat ', 'Parameters_DAonestep_Neumann.ini >> ',fcopt.simulation_dir,'Parameters.ini']);
    if fcopt.Dxx_opt == 0
        [~,~]=system(['cat ', 'Parameters_DAonestep_Neumann.ini Dxx_Fa.tmp >> ',...
            fcopt.simulation_dir,'Parameters.ini']);
    elseif fcopt.Dxx_opt >= 1
        [~,~]=system(['cat ', 'Parameters_DAonestep_Neumann.ini Dxx_Full.tmp >> ',...
            fcopt.simulation_dir,'Parameters.ini']);
    else
        error('unknown Dxx_opt\n');
    end

%% time and run VERB code
    cwd = pwd; %get current working directory
    code_time = tic;

    cd(fcopt.simulation_dir); %move to simulation directory
    [~,~]=system(['./',fcopt.verb_exe_name]);
    cd(cwd); 

%% load PSD
    PSD = load_plt([fcopt.simulation_dir,'Output/OutPSD.dat']);
    IPSD = PSD.arr(end,:,:,:);
    IPSD(IPSD < fcopt.minpsd) = fcopt.minpsd;

%% Return to current working dir
    tstr = format_time(toc(code_time));
    fprintf('VERB RUN TIME: %s\n',tstr);
