% run VERB and create matrix files
%
% written by A. C. Kellerman
%% save initial PSD for VERB computation
function omat = run_VERB_fc(fcopt,lastPSD,Kp_sim,Lpp_sim,mp_set)

    init_PSD_file = [fcopt.simulation_dir,'Input/da3d_init_psd.dat'];
    save_plt_lpa(init_PSD_file, lastPSD);

%% parameters structure:
    steps=1;
    fid = fopen([fcopt.simulation_dir,'Parameters.ini'], 'w');
    fprintf(fid, ['outputLvl = 3\n']);
    fprintf(fid, ['timeStep = ', num2str(fcopt.timestep,20), '\n']);
    fprintf(fid, ['nDays = ', num2str(fcopt.timestep.*1.00001./24.0,20), '\n']);
    fprintf(fid, ['general_Output_parameters.iterStep = ',num2str(steps,20), '\n']);

    fprintf(fid, 'useKp = constant\n');
    fprintf(fid, ['constKp = ', num2str(Kp_sim,20), '\n']);

    fprintf(fid, 'useLpp = constant\n');
    fprintf(fid, ['constLpp = ',num2str(Lpp_sim,20),'\n']);

    fprintf(fid, ['localDiffusionsGrid_L.size = ', num2str(fcopt.NL), '\n']);
    fprintf(fid, ['localDiffusionsGrid_L.min = ', num2str(fcopt.L_min), '\n']);
    fprintf(fid, ['localDiffusionsGrid_L.max = ', num2str(fcopt.L_max), '\n']);

    fprintf(fid, ['localDiffusionsGrid_epc.size = ', num2str(fcopt.NE), '\n']);
    fprintf(fid, ['localDiffusionsGrid_epc.min = ', num2str(fcopt.E_min), '\n']);
    fprintf(fid, ['localDiffusionsGrid_epc.max = ', num2str(fcopt.E_max), '\n']);

    fprintf(fid, ['localDiffusionsGrid_alpha.size = ', num2str(fcopt.NA), '\n']);
    fprintf(fid, ['localDiffusionsGrid_alpha.min = ', num2str(fcopt.a_min), '\n']);
    fprintf(fid, ['localDiffusionsGrid_alpha.max = ', num2str(fcopt.a_max), '\n']);
switch fcopt.upper_L
    case 'const0'
        fprintf(fid, ['L_UpperBoundaryCondition.type =  BCT_CONSTANT_VALUE\n']);
        fprintf(fid, ['L_UpperBoundaryCondition.value =  0\n']);  
    case 'Neuman'
        fprintf(fid, ['L_UpperBoundaryCondition.type =  BCT_CONSTANT_DERIVATIVE\n']);
        fprintf(fid, ['L_UpperBoundaryCondition.value =  0\n']);
    otherwise    
       fprintf(fid, ['L_UpperBoundaryCondition.type =  BCT_CONSTANT_PSD\n']); 
end
    %fprintf(fid, ['L_UpperBoundaryCondition.type =  BCT_CONSTANT_DERIVATIVE\n']);
    %fprintf(fid, ['L_UpperBoundaryCondition.value = 0\n']);
    
	fprintf(fid, ['L_LowerBoundaryCondition.type =  BCT_CONSTANT_VALUE\n']);
	fprintf(fid, ['L_LowerBoundaryCondition.value = 0\n']);
    
    fprintf(fid, ['pc_LowerBoundaryCondition.type = BCT_CONSTANT_PSD\n']); 
    
    fprintf(fid, ['pc_UpperBoundaryCondition.type = BCT_CONSTANT_VALUE\n']); 
    fprintf(fid, ['pc_UpperBoundaryCondition.value = 0\n']); 

    fprintf(fid, ['alpha_UpperBoundaryCondition.type = BCT_CONSTANT_DERIVATIVE\n']);
    fprintf(fid, ['alpha_UpperBoundaryCondition.value = 0\n']);

    %fprintf(fid, ['alpha_LowerBoundaryCondition.type = BCT_CONSTANT_DERIVATIVE\n']);        
    %fprintf(fid, ['alpha_LowerBoundaryCondition.value = 0\n']);
    fprintf(fid, ['alpha_LowerBoundaryCondition.type = BCT_CONSTANT_VALUE\n']);        
    fprintf(fid, ['alpha_LowerBoundaryCondition.value = 0\n']);        

    fprintf(fid, ['psdRadialDiffusion.initial_PSD_Type = IPSDT_FILE\n']);
    fprintf(fid, ['psdRadialDiffusion.initial_PSD_fileName = ./Input/da3d_init_psd.dat\n']);
    fprintf(fid, ['outputModelMatrix = true\n']);
    fprintf(fid, ['solveMatrix = false\n']);

    fprintf(fid, 'localDiffusionsGrid_alpha.useLogScale = No\n'); %default
    fclose(fid);

    if fcopt.Dxx_opt == 0
        [~,~]=system(['cat ', 'Parameters_DAonestep_Neumann.ini Dxx_Fa.tmp >> ',...
            fcopt.simulation_dir,'Parameters.ini']);
    elseif fcopt.Dxx_opt >= 1
        [~,~]=system(['cat ', 'Parameters_DAonestep_Neumann.ini Dxx_Full.tmp >> ',...
            fcopt.simulation_dir,'Parameters.ini']);
    else
        error('unknown Dxx_opt\n');
    end

  %  code_time = tic;
    cwd = pwd; %get current working directory

    %enter sim directory and run code
    cd(fcopt.simulation_dir);
    [~,~]=system(['./',fcopt.verb_exe_name]);
    cd(cwd); %return to current working dir

 %   tstr = format_time(toc(code_time));
 %   fprintf('VERB RUN TIME: %s\n',tstr);

    folder = [fcopt.simulation_dir,'Debug/'];

 %   loadtime=tic;

    omat.L_matr_A = load_matrix([folder,'L_matr_A.dat']);
    omat.L_matr_B = load_matrix([folder,'L_matr_B.dat']);
    omat.L_matr_C = load_matrix([folder,'L_matr_C.dat']);

    omat.pc_matr_A = load_matrix([folder,'pc_matr_A.dat']);
    omat.pc_matr_B = load_matrix([folder,'pc_matr_B.dat']);
    omat.pc_matr_C = load_matrix([folder,'pc_matr_C.dat']);

    omat.alpha_matr_A = load_matrix([folder,'alpha_matr_A.dat']);
    omat.alpha_matr_B = load_matrix([folder,'alpha_matr_B.dat']);
    omat.alpha_matr_C = load_matrix([folder,'alpha_matr_C.dat']);

%    tstr = format_time(toc(loadtime));
%    fprintf('MODEL MATRIX LOAD TIME: %s\n',tstr);

