%function used to load and process omni solar wind files
% Written by Adam Kellerman

function [sw_data, matlab_day] = get_SSA(year,varargin)

global dp_dirs

use_cache=false;
if nargin > 1
    for it=1:nargin-1
        if strcmp(varargin(it),'use_cache')
            use_cache=true;
            cache_dir=[getenv('HOME'),'/projects/cache/'];
            system(['mkdir -p ',cache_dir]);
        end
    end
end

ofdir = [dp_dirs.spc_data_dir,'SSA/processed_mat/'];
ofile = [num2str(year), '_SSA.mat']; 
matfile = dir([ofdir, ofile]); 
process = 0;
if ~isempty(matfile)
    %check date of omni.dat and omni.mat files, remake if dat is newer
    date_matfile = datenum(matfile.date);
    datfile = dir([dp_dirs.spc_data_dir,'SSA/WGhourFS_72_13.txt']);    

    date_datfile = datenum(datfile.date);
       
    if date_datfile > date_matfile
        fprintf('Dat file is newer: %s\n than mat file: %s\n',datfile.date,matfile.date);        
        process=1;
    else        
        fprintf('Dat file is older: %s\n than mat file: %s\n',datfile.date,matfile.date);
        fprintf('%s\n', ['Loading ',num2str(year), '_SSA.mat...']);
        if use_cache
            system(['rsync -avPzh ',ofdir,ofile,' ',cache_dir]);
            load([cache_dir,ofile]);
        else
            load([dp_dirs.spc_data_dir,'SSA/processed_mat/', num2str(year), '_SSA.mat']); 
        end
        return
    end
else
    process=1;
    fprintf('%s\n', ['no file']);
end
if process
    %%
    try
        parpool(4);
    catch
        fprintf('no parpool available or other error\n')
    end
    fprintf('%s', ['Searching for ','WGhourFS_72_13.txt','...']);
    txt_file = dir([dp_dirs.spc_data_dir,'SSA/WGhourFS_72_13.txt']);   
    if isempty(txt_file)
        fprintf('no asc file\n');
        sw_data = [];
        matlab_day = [];
    else
        fprintf('found\n');
        
        [ssa_data,ssa_time] = Load_SSA;
        
        Load_kp

        matlab_day = datenum(year-1,12,30):1/1440:datenum(year,12,31,23,59,59); 
        nt = length(matlab_day);
        sw_data = nan(nt,46);
        [yr,mo,dy,hr,mt,sc ] =datevec(matlab_day);
        sw_data(:,1) = yr;
        sw_data(:,2) = date2doy(matlab_day);
        sw_data(:,3) = hr;
        sw_data(:,4) = mt;
        sw_data(:,18) = interp1(ssa_time,ssa_data(:,1),matlab_day,'linear');
        sw_data(:,19) = interp1(ssa_time,ssa_data(:,2),matlab_day,'linear');
        sw_data(:,22) = interp1(ssa_time,ssa_data(:,3),matlab_day,'linear');
        sw_data(:,26) = interp1(ssa_time,ssa_data(:,4),matlab_day,'linear');
        sw_data(:,28) = interp1(ssa_time,ssa_data(:,5),matlab_day,'linear');
        Dst = interp1(ssa_time,ssa_data(:,12),matlab_day,'linear');

        
        Kp = interp1(kp_time, kp_index, matlab_day, 'nearest'); %in actual format
        %%
        
        fprintf('%s\n', 'Calculating G and W....');    
        make_G_and_W
        
        sw_data = cat(2, sw_data, Kp');  %47
        sw_data = cat(2, sw_data, Dst');
        sw_data = cat(2, sw_data, G1);
        sw_data = cat(2, sw_data, G2);
        sw_data = cat(2, sw_data, G3);
        sw_data = cat(2, sw_data, W1);
        sw_data = cat(2, sw_data, W2);
        sw_data = cat(2, sw_data, W3);
        sw_data = cat(2, sw_data, W4);
        sw_data = cat(2, sw_data, W5);
        sw_data = cat(2, sw_data, W6); %57
        
        fprintf('%s\n', 'done');
        %%
        fprintf('%s', 'Resizing array to year only..'); 
        zz=find(sw_data(:,1) == year);
        sw_data = sw_data(zz,:);
        matlab_day = matlab_day(zz);
        fprintf('%s\n', 'Done...');
        %%
        fprintf('%s', 'Saving....'); 
    %   save(['/home/akellerman/Documents/omni_data/', num2str(year), '_onemin.mat'], 'sw_data', 'matlab_day')    
        save([dp_dirs.spc_data_dir,'SSA/processed_mat/', num2str(year), '_SSA.mat'], 'sw_data', 'matlab_day')    
        fprintf('%s\n', 'done.');
    end
end
