function [PSD,Lstar_out] = interp2gridMK(mInvMu,mInvK,stime,sInvMu,sInvK,sPSD,sLstar,varargin);

% just interpolates spacecraft PSD to model Mu and K, and adjusts Lstar to match new K.
%mInvMu - model invariant mu
%mInvK - model InvK
%stime - spacecraft time
%sInvMu - spacecraft InvMu
%sInvK - spacecarft InvK
%sPSD - spacecraft PSD
%sLstar - spacecraft Lstar

method = 'natural'; %default
if nargin > 7
    it=1
    while it < nargin-7
        if strcmp(varargin(it),'method')
            method = varargin(it+1);
            it = it + 1;
        else
            error('unrecognized input argument')
        end
        it = it + 1;
    end
end

NE = size(mInvMu,1);
NA = size(mInvMu,2);
NT = length(stime);

PSD=nan(NT,NE,NA);
fprintf(' \n');
mod_InvMu = log10(reshape(mInvMu,1,[]));
mod_InvK = reshape(mInvK,1,[]).^(1/3);
Lstar_out = nan(NT,NA);
tmpK = mInvK(1,:).^(1/3);

sNE=size(sPSD,2);
sNA=size(sPSD,3);

sInvMu = log10(reshape(sInvMu,NT,[]));
sPSD = log10(reshape(sPSD,NT,[]));
sInvK = sInvK.^(1/3);

loopt = 1e6; %max elements for parpool loop

nloops = ceil(NT/loopt);
for loopi = 1:nloops

    inxstart = (loopi - 1) * loopt + 1;
    inxend = inxstart + loopt - 1;
    if inxend > NT
        inxend = NT;
    end
    sInvMu_t = sInvMu(inxstart:inxend,:);
    sPSD_t = sPSD(inxstart:inxend,:);
    sInvK_t = sInvK(inxstart:inxend,:);
    sLstar_t = sLstar(inxstart:inxend,:);

    endloop = inxend - inxstart + 1;

    Lstar_out_t = nan(endloop,NA);
    PSD_t = nan(endloop,NE,NA);

%    looptxt = sprintf('loop %i/%i, inx: ',loopi,nloops);
    parfor it=1:endloop
       % loop_counter(it,endloop,'tput','mod',1000,'pretext',looptxt);

        spc_InvMu = sInvMu_t(it,:);
        spc_PSD = sPSD_t(it,:);
        nn = find(~isnan(sInvK_t(it,:)) & ~isnan(sLstar_t(it,:)));
        if length(nn) > 1
            si = sInvK_t(it,nn);
            sl = sLstar_t(it,nn);
            [siu,ia,ic] = unique(si);
            if length(siu) < length(si)
                sl = sl(ia);
                si=siu;
            end
            if length(si) > 1
                Lstar_out_t(it,:) = interp1(si,sl,tmpK,'linear',NaN);
                zz=find(isnan(Lstar_out_t(it,:)));
                zz2=find(~isnan(Lstar_out_t(it,:)));
                tmpL = Lstar_out_t(it,:);                
                if ~isempty(zz) && length(zz2) > 1
                    Lstar_out_t(it,:) = interp1(tmpK(zz2),tmpL(zz2),tmpK,'nearest','extrap');
                elseif length(zz2) == 1
                    Lstar_out_t(it,:) = tmpL(zz2); %use the one point, ignore d-shell splitting
                end
            elseif length(si) == 1
                Lstar_out_t(it,:) = sl; % use the one point and ignore d-shell splitting
            end
        elseif length(nn) == 1
            Lstar_out_t(it,:) = sLstar(it,nn);
        end

        spc_InvK = nan(sNE,sNA);
        for ie=1:sNE
            spc_InvK(ie,:) = sInvK_t(it,:);
        end     
        spc_InvK = reshape(spc_InvK,1,[]);

        nn = find(~isnan(spc_InvMu) & ~isnan(spc_InvK) & ~isnan(spc_PSD));
        if ~isempty(nn)
            spc_InvK = spc_InvK(nn);
            spc_InvMu = spc_InvMu(nn);
            spc_PSD = spc_PSD(nn);

            PSD_out = griddata(spc_InvMu,spc_InvK,spc_PSD,...
                mod_InvMu, mod_InvK,method);
            if ~isempty(PSD_out)
                PSD_out = reshape(PSD_out,1,NE,NA);
                PSD_t(it,:,:) = PSD_out;
            end
        end
    end
    PSD(inxstart:inxend,:,:) = PSD_t;
    Lstar_out(inxstart:inxend,:,:) = Lstar_out_t;
end
PSD = 10.^(PSD);

