function [res, absdev] =  ladfit(xIn, yIn)
   nX = length(xIn);

   if nX ~= length(yIn) 
        MESSAGE = 'X and Y must be vectors of equal length.'
   end 
     
   x = xIn;
   y =yIn;

   sx = sum(x);
   sy = sum(y);

   sxy = sum(x.*y);
   sxx = sum(x.*x);
   del = nX*sxx - sx^2;

  if (del == 0.0)           %%All X's are the same
    result = [nanmedian(y), 0.0]; %%%Bisect the range w/ a flat line
    return
  end

 
  aa = (sxx * sy - sx * sxy)/del; %%%Least squares solution y = x * aa + bb
  bb = (nX * sxy - sx * sy)/del;
  chisqr = sum((y - (aa + bb*x)).^2);
  sigb = sqrt(chisqr/del);     %%%Standard deviation

  b1 = bb;
  eps = 1d-7;
  [f1, aa, absdev] = MDfunc(b1, x, y, aa, eps);


  %%%Quick return. The initial least squares gradient is the LAD solution.
  if (f1 == 0.) 
    bb = b1;
    absdev = absdev/nX;
    return
  end

   
   if (f1>=0)
       delb = 3.0 * sigb;
   else
       delb = -3.0 * sigb;
   end    
       
  
   b2 = b1 + delb;
   [f2, aa, absdev] = MDfunc(b2, x, y, aa, eps);
  
       
  while (f1*f2 > 0)  %%%Bracket the zero of the function
      b1 = b2;
      f1 = f2;
      b2 = b1 + delb;
      [f2, aa, absdev] = MDfunc(b2, x, y, aa, eps);
  end

  %%%In case we finish early.
  bb = b2;
  f = f2;
  
%%%Narrow tolerance to refine 0 of fcn.
 sigb = 0.01*sigb;

  while ((abs(b2-b1) > sigb) && (f ~= 0))  %%%bisection of interval b1,b2.
    bb = 0.5*(b1 + b2);
    if ((bb == b1) || (bb == b2)) 
        break
    end    
    [f, aa, absdev] = MDfunc(bb, x, y, aa, eps);
    if (f*f1 >= 0) 
        f1 = f;
        b1 = bb;
   else 
        f2 = f;
        b2 = bb;
    end
  end
 
  absdev = absdev/nX;
  res = [aa, bb];
  return
end
