%%%% Written by Adam Kellerman

%%make_G_and_W, lsw_dat% B_X to AE
%format long;

%lsw_data format
% 1  -  year
% 2  -  doy
% 3  -  month
% 4  -  day
% 5  -  hour
% 6  -  minute
% 7  -  second
% 8  -  total time in second since Jan 1, 1970
% 9  -  altitude in km
% 10 - geographic latitude
% 11 - geographic longitude  (GDZ coordinate)
% 12 - BxIMF
% 13 - ByIMF
% 14 - BzIMF
% 15 - SWSpeed
% 16 - SWDensity
% 17 - SWPressure
% 18 - Dst_tmp
% 19 - Kp
% 20 - AE
% 21 - 23 G1 - G3
% 24 - 29 W1 - W6
clear G1
clear G2
clear G3
clear W1
clear W2
clear W3
clear W4
clear W5
clear W6

% Year=SW(:,1);
% Doy=SW(:,2);
% Hour=SW(:,3);
% Min=SW(:,4);

By=sw_data(:,18);
Bz=sw_data(:,19);
Vsw=sw_data(:,22);
Nsw=sw_data(:,26);
Pdyn=sw_data(:,28);
%AE=sw_data(:,38);
%Kp=lsw_dat(:,19);
%Dst_tmp=lsw_dat(:,18);
[ss1,tt1]=size(sw_data);
%%
% parameters for T2001 models
G1(1:ss1,1)=NaN;
G2(1:ss1,1)=NaN;
G3(1:ss1,1)=NaN;

% parameters for T2001 storm model
W1(1:ss1,1)=NaN;
W2(1:ss1,1)=NaN;
W3(1:ss1,1)=NaN;
W4(1:ss1,1)=NaN;
W5(1:ss1,1)=NaN;
W6(1:ss1,1)=NaN;

% for T2001 models
clear Bperp
clear h_bperp
clear ind1
clear theta
clear Bs
clear ind2
clear ind3

theta(1:ss1,1)=NaN;
Bperp(1:ss1,1)=NaN;
h_bperp(1:ss1,1)=NaN;
Bs(1:ss1,1)=NaN;

Bperp(:,1)=sqrt(Bz.*Bz+By.*By);
h_bperp(:,1)=(Bperp./40.0).*(Bperp./40.0)./(1.0+Bperp./40.0);
ind1=find(Bz>0);
theta(ind1,1)=atan(abs(By(ind1))./Bz(ind1));
Bs(ind1,1)=0.0;
ind2=find(Bz==0);
theta(ind2,1)=0.5;
Bs(ind2,1)=0.0;
ind3=find(Bz<0);
theta(ind3,1)=pi-atan(abs(By(ind3))./Bz(ind3));
Bs(ind3,1)=-Bz(ind3);
if ~isempty(theta)
    if length(theta) == length(Bz)
        
        G1_series=Vsw.*h_bperp.*sin(theta./2.0).*sin(theta./2.0).*sin(theta./2.0);
        G2_series=Vsw.*Bs./200.0;
        G3_series=Nsw.*Vsw.*Bs./2000.0;
        
        % for T2010 storm model
        r1=0.39; d1=0.39; b1=0.80; g1=0.87;
        r2=0.70; d2=0.46; b2=0.18; g2=0.67;
        r3=0.031;d3=0.39; b3=2.32; g3=1.32;
        r4=0.58; d4=0.42; b4=1.25; g4=1.29;
        r5=1.15; d5=0.41; b5=1.60; g5=0.69;
        r6=0.88; d6=1.29; b6=2.40; g6=0.53;
        
        S1=(Nsw./5.0).^d1.*(Vsw./400.0).^b1.*(Bs./5.0).^g1;
        S2=(Nsw./5.0).^d2.*(Vsw./400.0).^b2.*(Bs./5.0).^g2;
        S3=(Nsw./5.0).^d3.*(Vsw./400.0).^b3.*(Bs./5.0).^g3;
        S4=(Nsw./5.0).^d4.*(Vsw./400.0).^b4.*(Bs./5.0).^g4;
        S5=(Nsw./5.0).^d5.*(Vsw./400.0).^b5.*(Bs./5.0).^g5;
        S6=(Nsw./5.0).^d6.*(Vsw./400.0).^b6.*(Bs./5.0).^g6;
        
        % compute W params, W require specific 5-min input.
		time_5 = (datenum(year,1,1) - 90):5/1440:datenum(year+1,1,1);
		sw_5 = [];
		inx = [19 22 26];
		for iinx=1:length(inx);
        	sw_5(:,iinx) = interp1(matlab_day,sw_data(:,inx(iinx)),time_5,'linear','extrap');
		end

		Dst_5 = interp1(matlab_day,Dst,time_5,'linear',NaN);

		%Bz_5=sw_5(:,19)';
		%Vsw_5=sw_5(:,22)';
		%Nsw_5=sw_5(:,26)';
		Bz_5=sw_5(:,1)';
		Vsw_5=sw_5(:,2)';
		Nsw_5=sw_5(:,3)';

		% Coefficients for parameterization
		lambda  =   [0.39,  0.46,   0.39,   0.42,   0.41,   1.29];
		beta    =   [0.8,   0.18,   2.32,   1.25,   1.6,    2.4];
		gamma   =   [0.87,  0.67,   1.32,   1.29,   0.69,   0.53];
		r       =   [0.39,  0.7,    0.031,  0.58,   1.15,   0.88];

		Dst_storm = -40; %nT
		storm_idx = find(Dst_5 < Dst_storm);

		storm_idx_filtered = storm_idx(1);
		for i = 2:length(storm_idx)
			if storm_idx(i) > storm_idx(i-1) + 12
				storm_idx_filtered = cat(1, storm_idx_filtered, storm_idx(i));
			end
		end

		% redine storm idx here to represent start of storm (when DST is less than 0), prior to threshold setting.
		storm_idx = [];
		for i = 1:length(storm_idx_filtered)
			current_storm_idx = storm_idx_filtered(i);
			Dst_cur = Dst_5(current_storm_idx);
			while (Dst_cur < 0)
				current_storm_idx = current_storm_idx - 1;
				if current_storm_idx >= 1
					Dst_cur = Dst_5(current_storm_idx);
				else
					current_storm_idx = 1;
					break;
				end
			end
			storm_idx = cat(1, storm_idx, current_storm_idx);
		end

		% Filter out repeating indices of the storm beginning:
		storm_idx = unique(storm_idx);

		% Add last index to iterate over storm_idx later
		storm_idx = cat(1, storm_idx, length(time_5)+1);

		% Calculate W1-W6 coefficients
		fprintf('computing W coefficients...\n\n')
		W_5 = nan(6, length(time_5));
		Bs_5 = abs(Bz_5) .* (Bz_5 < 0);
		idx_s = storm_idx(1);
		storm_idx = storm_idx(2:end);
		inx=1;
		for idx_e = storm_idx'
			loop_counter(inx,length(storm_idx),'tput');
			for idx_c = idx_s:idx_e-1
				for i = 1:6
					S_tmp = (Nsw_5(idx_s:idx_c) / 5.  ) .^ lambda(i) .* ...
							(Vsw_5(idx_s:idx_c) / 400.) .^ beta(i)   .* ...
							(Bs_5(idx_s:idx_c)  / 5.  ) .^ gamma(i);
					W_5(i,idx_c) = r(i) ./ 12 .* sum(S_tmp .* exp(r(i) .* ...
						(time_5(idx_s:idx_c) - time_5(idx_c)) ./ 60 * 24. * 60.));
					% An attempt to take into account hysteresis effect:
					if idx_s ~= 1
						W_prev = W_5(i, idx_s-1);
					else
						W_prev = W_5(i, idx_s);
					end
					if isnan(W_prev)
						W_prev = W_5(i,idx_s);
					end
					hysteresis_time = 6/24; % in days
					W_5(i,idx_c) = W_5(i,idx_c) + ...
						W_prev .* exp(-(time_5(idx_c) - time_5(idx_s))/hysteresis_time);
				end
			end
			idx_s = idx_e;
			inx=inx+1;
		end
		fprintf('done\n');
		
		W = interp1(time_5,W_5',matlab_day,'linear',NaN);

		for i=1:6
			eval(['W',num2str(i),' = W(:,i);']);
		end

        %% compute G params
		fprintf('computing G coefficients...')
        
        parfor i=61:ss1
            % calculate G1 - averaging over the preceding 1-hour interval
            % (Tsyganenko, 2010, JGR, 107(A8), 1176, equation (1))
            G1(i,1)=nanmean(G1_series((i-60):(i)));
            % calculate G2 - averaging over the preceding 1-hour interval
            % (Tsyganenko, 2010, JGR, 107(A8), 1176, equation (2) or Tsyganenko et al., 2010, JGR, 108(A5), 1209, below equation (1))
            G2(i,1)=nanmean(G2_series((i-60):(i)));
            % calculate G3 - averaging over the preceding 1-hour interval
            % (Tsyganenko et al., 2010, JGR, 108(A5), 1209, below equation (2))
            G3(i,1)=nanmean(G3_series((i-60):(i)));
                   
        end
		fprintf('done\n')
    end
end
