% written by A. Kellerman, 2014

function [maginput] = make_maginput_hist(matlabd,varargin)

% uses both realtime and historic data, depending on availability
global dp_dirs

use_cache=false;
if nargin > 1
    for it=1:nargin-1
        if strcmp(varargin(it),'use_cache')
            use_cache=true;
        end
    end
end
nT = length(matlabd);

st = matlabd(1);
et = matlabd(end);

[syr,~,~,~,~,~] = datevec(st);
[eyr,~,~,~,~,~] = datevec(et);

sw_d=[];
sw_t=[];
if syr ~= eyr
    for iyr = syr:eyr
        if (use_cache)
            [sw_data, sw_matlabd]=get_omnimin(iyr,'use_cache');
        else
            [sw_data, sw_matlabd]=get_omnimin(iyr);
        end
        sw_d = cat(1,sw_d,sw_data);
        sw_t = cat(1,sw_t,sw_matlabd);
    end
    sw_data = sw_d;
    sw_matlabd = sw_t;
    clear sw_d sw_t
else
    if (use_cache)
        [sw_data, sw_matlabd]=get_omnimin(syr,'use_cache');
    else
        [sw_data, sw_matlabd]=get_omnimin(syr);
    end
end

zz=find(sw_matlabd >= st - 5/1440 & sw_matlabd <= et + 5/1440);

sw_data = sw_data(zz,:);
sw_matlabd = sw_matlabd(zz);

maginput=nan(25,nT);

% treat kp separately
Kpout = Load_Kp;
Kp = interp1(Kpout.kp_time,Kpout.kp_index,matlabd,'nearest',NaN);
Kp(Kp < 0) = 0.0;
maginput(1,:) = Kp;

if length(matlabd) > 1
    dt = median(diff(matlabd))/2.;
else
    dt = 1/1440;
end

tmplsw = nan(7,nT);
parfor it=1:nT
    loop_counter(it,nT,'tput','mod',1000,'pretext','maginput, mag plasma: ');
    svar=0;
    zz=find(sw_matlabd >= matlabd(it)-dt & sw_matlabd < matlabd(it)+dt);
    if ~isempty(zz)
        tmpvars = nan(7,1);
        for ivar=1:7

            if ivar == 7
                svar=15; %Bx, GSM

            elseif ivar == 5
                svar=18; %By, GSM

            elseif ivar == 6
                svar=19; %Bz, GSM

            elseif ivar == 3
                svar=22; %Flow speed, km/s

            elseif ivar == 2
                svar=26; %Proton Density, n/cc

            elseif ivar == 4
                svar=28; %Flow Pressure, nPa

            elseif ivar == 1
                svar=48; %Sym-H converted to Dst / Forecast Dst
            end
    
            tmpvals = sw_data(zz,svar);
            zz2=find(~isnan(tmpvals) & tmpvals ~=-1e31);
            if ~isempty(zz2)
                %lsw_dat(it,ivar+11) = mean(tmpvals(zz2));
                tmpvars(ivar) = mean(tmpvals(zz2));
            end
        end
        tmplsw(:,it) = tmpvars;
    end
end
maginput(2:7,:) = tmplsw(1:6,:);
clear tmplsw

tmplsw = nan(1,nT);
parfor it=1:nT
    loop_counter(it,nT,'tput','mod',1000,'pretext','maginput,AL: ');
    zz=find(sw_matlabd >= matlabd(it)-dt & sw_matlabd < matlabd(it)+dt);
    if ~isempty(zz)
        tmpvals = sw_data(zz,39); %AL
        zz2=find(~isnan(tmpvals) & tmpvals ~=-1e31);
        if ~isempty(zz2)
            tmplsw(it) = mean(tmpvals(zz2));
        end
    end
end
maginput(17,:) = tmplsw;
clear tmplsw

%tmplsw = nan(9,nT);
tmplsw = interp1(sw_matlabd,sw_data(:,49:57),matlabd,'nearest',NaN);

maginput(8:16,:) = tmplsw';
maginput=double(maginput);
clear tmplsw
