function [flux_time,Flux,xGEO,ech] = nflux_to_flux(spc_base_dir,nflux_file,ftag,mission,spc,instrLKB,dates);

pa_dir = [spc_base_dir,'Pitch_Angles/'];

load(nflux_file);

% note special case for LANL_GEO and Cluster spacecraft
if strcmp(instrLKB,'SOPAceval') || strcmp(instrLKB,'RAPID')
    %pa_file = [pa_dir,spc,'_',instrLKB,'_PitchAngles_',dates,'.txt.gz'];
    pa_file = [pa_dir,spc,'_',instrLKB,'_PAHead_',dates,'.txt.gz'];
    fck_pafile = dir(pa_file);
    if isempty(fck_pafile)
        fprintf('%s%s\n','GEO_LANL and Cluster spacecraft dont have pitch-angle resolved data',...
        'so we compute this based on the spin-averaged fluxes')
        error('No pitch angle files were found. These are required to ensure the L* computations match the values computed here.');
    end

    system(['cp ',pa_file,' tmp/',ftag,'pa.txt.gz']);
    tmpdir = 'tmp/';
    system(['gunzip tmp/',ftag,'pa.txt.gz']);
    tfile = ['tmp/',ftag,'pa.txt'];

    fileID = fopen(tfile);
    C = textscan(fileID,'%f');
    fclose(fileID);
    D = cell2mat(C);
    local_pitch_angles = nan(D(1),D(2));
    idx=3;
    for it=1:D(1)
        for ipa=1:D(2)
            local_pitch_angles(it,ipa) = D(idx);
            idx = idx + 1;
        end
    end
            
    system(['rm tmp/',ftag,'pa.txt']);

    % now let's create a PAD based on the spin-averaged fluxes.
    % note that there are likely to be errors associated with this.
    sz = size(localflux);
    newflux = nan(sz(1),sz(2),length(local_pitch_angles));

    % if we have spin averaged data then the value at 90 will be higher than the spin average; for a sin distribution.
    factor = 1 ./ mean(sin(0:pi/48:pi));
    
    for it=1:sz(1)
        for ie=1:sz(2)
            newflux(it,ie,:) = localflux(it,ie).* sin(local_pitch_angles.*pi/180) .* factor;
        end
    end
    localflux = newflux;
    clear newflux
end
Flux = localflux;


nt = length(flux_time);

% ensuring that local pitch angles is the correct shape
if size(local_pitch_angles,2) == nt 
    local_pitch_angles = local_pitch_angles';
elseif size(local_pitch_angles,1) > 1 && ...
    size(local_pitch_angles,1) < nt
    local_pitch_angles = local_pitch_angles';
end                                

sz =size(local_pitch_angles);
if sz(1) > 1 && length(size(Flux)) < 3 && size(Flux,2) ~= sz(2)
    % then we have an issue in the original nflux file (Lomonosov is like this) the pitch angle
    % info was dumped from the localflux variable as it was size 1, even though there was one measured pitch angle

    fprintf('* nflux file with pitch angle data detected, but no pitch angle dimension in localflux variable\n')
    if sz(2) > 1
        error('localflux variable should have size: %i %i %i, but has size: %i %i\n',size(Flux,1),size(Flux,2),...
            sz(2),size(Flux,1),size(Flux,2));
    else
        fprintf('# adding dimesion of size %i\n',sz(2)); %should be 1, but we use the size to check
        pause(1)
        newFlux = nan(size(Flux,1),size(Flux,2),sz(2));
        newFlux(:,:,sz(2)) = Flux;
        Flux = newFlux;
    end
end

na = size(local_pitch_angles,2);

if (MeV)  %if units are in MeV
    Flux=Flux/1000; %convert to /(cm^2.s.sr.keV)
end
% let's convert position data to xGEO
A = exist('xGEO','var');
if A ~= 1
    fprintf('# determining position type...');
    B = exist('xGDZ','var');
    C = exist('xGSM','var');
    D = exist('xGSE','var');
    E = exist('xSM','var');
    F = exist('xGEI','var');
    G = exist('xMAG','var');
    H = exist('xRLL','var');
    if B == 1
        coord = 'xGDZ';
        fprintf('converting %s to xGEO\n',coord)
        xGEO = onera_desp_lib_coord_trans(xGDZ,'GDZ2GEO',flux_time);
    elseif C == 1
        coord = 'xGSM';
        fprintf('converting %s to xGEO\n',coord)
        xGEO = onera_desp_lib_coord_trans(xGSM,'GSM2GEO',flux_time);
    elseif D == 1
        coord = 'xGSE';
        fprintf('converting %s to xGEO\n',coord)
        xGEO = onera_desp_lib_coord_trans(xGSE,'GSE2GEO',flux_time);
    elseif E == 1
        coord = 'xSM';
        fprintf('converting %s to xGEO\n',coord)
        xGEO = onera_desp_lib_coord_trans(xSM,'SM2GEO',flux_time);
    elseif F == 1
        coord = 'xGEI';
        fprintf('converting %s to xGEO\n',coord)
        xGEO = onera_desp_lib_coord_trans(xGEI,'GEI2GEO',flux_time);
    elseif G == 1
        coord = 'xMAG';
        fprintf('converting %s to xGEO\n',coord)
        xGEO = onera_desp_lib_coord_trans(xMAG,'MAG2GEO',flux_time);
    elseif H == 1
        coord = 'xRLL';
        fprintf('converting %s to xGEO\n',coord)
        xGEO = onera_desp_lib_coord_trans(xRLL,'RLL2GEO',flux_time);
    else
        who
        error(['unrecognized or non-existant position data,',...
                'should be a variable of the form: xZZZ'])
    end
end
