% POES create L* and B from MATLAB
% Author A. Kellerman
%profile on

tic
clear
if (~isdeployed)
    addpath([getenv('DATA_ORG_DIR'),'/Functions/']);
end
data_dir = ([getenv('SAT_DATA_DIR'),'/']);

%% SETTINGS:

kext = 4; % external field model, 4 = T89
options = [1 1 4 4 0]; % controls IRBEM parameters: internal field model (IGRF), refresh time, and tracing accuracy, see doc
sysaxes = 1; %1 - GEO POES coordinates saved in the Nflux files
recalc = false;
%start_year = 2013;
%end_year = 2017;
start_year = 2017;
end_year = 2017; % testing (one year only)
int = true; %default false, for POES it's true, see below
missions = {'POES'};

%ncpu=16; % standart
%ncpu=24; % rbm7 (max is 128)
ncpu=32; % rbm7 (max is 128)
%ncpu=40; % rbm7 (max is 128)
%ncpu=48; % rbm7 (max is 128)
%ncpu=56; % rbm7 (max is 128)
%ncpu=64; % rbm7 (max is 128)
%ncpu=128; % rbm7 (max is 128) - very slow


% parpool(ncpu)
toc
tic

%%%%%%%%% END SETTINGS %%%%%%%%%%%%

Ini_dirs('set_numwrk',ncpu);
pp_flag = true;

nmission = length(missions);
[mfm] = get_mfm(kext);

for imission = 1:nmission
    mission = missions{imission}

    for iyr = start_year:end_year
        year=num2str(iyr);
       
        if strcmp(mission,'POES')
            searchdir1=[data_dir,'POES/OriginalData/full/',year,'/'];
            % interp observations for poes
            int = true;
            %int_iv = 15; %seconds 
            int_iv = 60; % 1 minute
			%int_iv = 60*60; % 60 minuts (for testing)
        else
            error('mission not added here yet')
        end
        sats = dir([searchdir1,'*']);
        sats = sats(3:end); % Exclude '.' and '..'

        %sats = struct('name','noaa15'); % limit to only one entry
        nsats = length(sats);
		disp(sats);
        for isat=1:nsats
%        for isat=1:nsats % last sat
%        for isat=1:nsats % last sat
%        for isat=1 % last sat
            sat=sats(isat).name;
            nfdir = [data_dir,mission,'/',sat,'/Nflux/'];
            bldir = [data_dir,mission,'/',sat,'/BLdat/'];

            for imo=4:4 % only april
%            for imo=1:1 % one month
                dates = make_dates(iyr,imo);
                fprintf('%s %s...',sat,dates)

                infile = [nfdir,sat,'_SEM2_Nflux_',dates,'.mat'];
                fck1 = dir(infile);
                outfile = [bldir,sat,'_SEM2_BL_',dates,'_',mfm,'.mat'];
                fck2 = dir(outfile);

                process = false;
                nonf = false;
                clear Flux xGEO lpa
                if ~isempty(fck2)
                    load(outfile,'xGEO');
                    load(outfile,'lpa');
                    if ~exist('xGEO','var') || ~exist('lpa','var')
                        nonf = true;
                        fprintf('recomputing to add stuff\n')
                    end
                    if fck2.datenum < fck1.datenum
                        process = true;
                    end
                else
                    process = true;
                end

                if process || recalc || nonf
                    if isempty(fck1)
                        fprintf('no nflux file\n')
                    else
                        fprintf('processing\n');

                        fprintf('loading nflux...');
                        data = load(infile);
                        fprintf('done\n');


                        % make ldat and bdat-type files
                        % get the solar wind for the period
                        st = datenum(iyr,imo,1) - 1;
                        et = datenum(iyr,imo+1,1) + 1;
                        [~,Kp] = omniweb(st,et,38);

                        x1 = data.xGEO(:,1);
                        x2 = data.xGEO(:,2);
                        x3 = data.xGEO(:,3);

                        if int
                            time = data.flux_time;
                            lpa = data.local_pitch_angles;

                            newtime = [st:int_iv/86400:et]'; % time must be column (time x 1), not a row!
                            x1 = interp1(time,x1,newtime,'nearest');
                            x2 = interp1(time,x2,newtime,'nearest');
                            x3 = interp1(time,x3,newtime,'nearest');
                            lpa = interp1(time,data.local_pitch_angles,newtime,'nearest');
                            Flux = interp1(time,data.localflux,newtime,'nearest');
                            time = newtime;
                            clear data newtime
                        else
                            time = data.flux_time;
                            Flux = data.localflux;
                            lpa = data.local_pitch_angles;
                        end

                        % Vector check
                        if (size(x1, 1) > size(x1, 2))
                            xGEO = cat(2,x1,x2,x3);
                        else
                            x1 = x1';
                            x2 = x2';
                            x3 = x3';
                            xGEO = cat(2,x1,x2,x3);
                        end

                        if nonf % then just save the Flux variable
                            save(outfile,'xGEO','lpa','-append');
                        else

                            % create maginput array:
                            nt = length(time);

                            maginput = nan(nt,25); % see doco for other array elements
                            %maginput(:,1) = interp1(Kp.time,Kp.arr/10,time,'nearest'); %interpolate using nearest
                            maginput(:,1) = interp1(Kp.time,Kp.arr,time,'nearest'); %interpolate using nearest

                            max_size = 30000; %max size parallized / cpu (sent to fortran)

                            na = size(lpa,2);

                            % pre-allocate our B-field arrays
                            B_local = nan(nt,1);
                            B_eq = nan(nt,1);
                            MLT = nan(nt,1);
                            Lstar = nan(nt,na);
                            Lm = nan(nt,na);
                            InvI = nan(nt,na);
                            Bmir = nan(nt,na);
                            a_eq = nan(nt,na);

                            % break it into parts
                            sinx = 1;
                            einx = min([max_size nt]) ;
                            
                            fprintf('computing B, L*\n');
                            tic

                            while sinx <= nt
                                nt2 = einx - sinx + 1;

                                ttmp = time(sinx:einx);
                                x1tmp = x1(sinx:einx);
                                x2tmp = x2(sinx:einx);
                                x3tmp = x3(sinx:einx);
                                maginput_tmp = maginput(sinx:einx,:);
                                patmp = lpa(sinx:einx,:);

                                Bl = nan(nt2,1);
                                Beq = nan(nt2,1);
                                mlt = nan(nt2,1);
                                L = nan(nt2,na);
                                L2 = nan(nt2,na);
                                aeq = nan(nt2,na);
                                Bm = nan(nt2,na);
                                I = nan(nt2,na);

                                if pp_flag
                                    try
                                        parpool(ncpu);
                                    catch end
                                    pp_flag = false;
                                end


                                % use parallal processing
                                 parfor it=1:nt2                       
%                                 super_range = 10;
%                                 for it=1:super_range:nt2
                                  % it2 = min(it + super_range - 1, nt2);
                                    if mod(it,100) == 0
                                        fprintf('B,L* - total: %i, %i-%i: %i\n',nt,sinx,einx,it);
                                    end
                                    
%                                     itrange = it:it2;
%                                     inan = isnan(x1tmp(itrange)) | isnan(x2tmp(itrange)) | isnan(x3tmp(itrange));
%                                     itrange = itrange(~inan);
%                                     if ~all(inan)
%                                         [~,B1] = onera_desp_lib_get_field(kext,options,sysaxes,ttmp(itrange),x1tmp(itrange),...
%                                             x2tmp(itrange),x3tmp(itrange),maginput_tmp(itrange,:));
%                                         Bl(itrange) = B1;
%                                         for ia=1:na
%                                             alpha = patmp(itrange,ia);
%                                             [A2,B2,C2,D2,E2,F2] = onera_desp_lib_make_lstar_shell_splitting(...
%                                                 kext,options,sysaxes,ttmp(itrange),x1tmp(itrange),x2tmp(itrange),x3tmp(itrange),alpha',maginput_tmp(itrange,:));
% 
%                                             L2(itrange,ia) = A2;
%                                             L(itrange,ia) = B2;
%                                             Bm(itrange,ia) = C2;
%                                             Beq(itrange) = D2;
%                                             I(itrange,ia) = E2;
%                                             mlt(itrange) = F2;
%                                             if D2 > B1
%                                                 D2 = B1; %probably tracing accuracy limit.
%                                             end
%                                             aeq(itrange,ia) = asin(sqrt(D2./B1 .* sin(alpha.*pi/180).^2)); 
%                                         end
%                                     end
                                                                        
                                    if ~isnan(x1tmp(it)) && ~isnan(x2tmp(it)) && ~isnan(x3tmp(it))
                                        [~,B1] = onera_desp_lib_get_field(kext,options,sysaxes,ttmp(it),x1tmp(it),...
                                            x2tmp(it),x3tmp(it),maginput_tmp(it));
                                        Bl(it) = B1;

                                        for ia=1:na
                                            alpha = patmp(it,ia);
                                            [A2,B2,C2,D2,E2,F2] = onera_desp_lib_make_lstar_shell_splitting(...
                                                kext,options,sysaxes,ttmp(it),x1tmp(it),x2tmp(it),x3tmp(it),alpha,maginput_tmp(it,:));

                                            L2(it,ia) = A2;
                                            L(it,ia) = B2;
                                            Bm(it,ia) = C2;
                                            Beq(it) = D2;
                                            I(it,ia) = E2;
                                            mlt(it) = F2;
                                            if D2 > B1
                                                D2 = B1; %probably tracing accuracy limit.
                                            end
                                            aeq(it,ia) = asin(sqrt(D2./B1 .* sin(alpha.*pi/180).^2)); 
                                        end
                                    end
                                end
                                
                                Lm(sinx:einx,:) = L2;
                                Lstar(sinx:einx,:) = L;
                                Bmir(sinx:einx,:) = Bm;
                                B_eq(sinx:einx) = Beq;
                                InvI(sinx:einx,:) = I;
                                MLT(sinx:einx) = mlt;

                                B_local(sinx:einx) = Bl;
                                a_eq(sinx:einx,:) = aeq;

                                sinx = einx + 1;
                                einx = sinx + max_size - 1;
                                einx = min([einx nt]) ;
                            end
                            telap = toc;
                            persec =  telap / nt;
                            BL_time = time;
                            [~,~] = mkdir(bldir);
                            save(outfile,'BL_time','B_local','B_eq','MLT','Lstar','Lm',...
                                'a_eq','Flux','lpa','InvI','Bmir','xGEO','telap','persec');
                        end
                    end
                else
                    fprintf('ok\n');
                end
				toc
            end        
		end
    end
end


