% Special version of data.org 1 files, used to load MATLAB computed L and B data
%
% Written by Adam C. Kellerman, July 2018
% questions should be addressed directly to the Author.

clear
if (~isdeployed)
    addpath([getenv('DATA_ORG_DIR'),'/Functions/']);
end
Ini_dirs('set_numwrk',8);

global dp_dirs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% PMF settings - also see get_mission_spc_instr script %%%%%%%%%%%%%%%%%%%%%%%%%%
%% setup MF models to run, default is all
mfms = {'T89'};

%% date setting (will skip times with no nflux files)
start_year=2013;
stmo=1;

[end_year,enmo,~,~,~,~] = datevec(now);

% add any additional spacecraft here:

missions = {'POES','POES','POES','POES','POES','POES','POES'};
spcs = {'metop01','metop02','noaa15','noaa16','noaa17','noaa18','noaa19'};
instrs = {'SEM2','SEM2','SEM2','SEM2','SEM2','SEM2','SEM2'};
n3 = 4;
n4 = 4;
proc_vers = 'ver4';

%% constants
mc2 = 0.511;
load_lstar_matfile = true; %for POES, need to check these before computing again

nspc = length(spcs);
nmfm= length(mfms);

n_str=sprintf('n%s_%s',num2str(n3),num2str(n4));

for ispc=1:nspc
    mission = missions{ispc};
    spc = spcs{ispc};
    instr = instrs{ispc};

    % in a few cases, we don't need to compute L* files for different Nflux files/instr files
    % this is relevant to MAGEIS corrected/non corrected data, and LANL_GEO ceval/feval data
    % in both cases the position and pitch angle data are the same, so L* computations are identical.
    % we use the instrLKB variable to define the 'base' instrument string associated with the L* comps.

    spc_base_dir = [dp_dirs.spc_data_dir,mission,'/',spc,'/'];
    lb_dir = [spc_base_dir,'BLdat/']; % we use the same directory

    for iyr=start_year:end_year
        if iyr == start_year
            smo=stmo;
        else
            smo=1;
        end
        if iyr == end_year
            emo=enmo;
        else
            emo=12;
        end
        for imo=smo:emo
            dates = make_dates(iyr,imo);

            nf_file = [spc_base_dir,'Nflux/',spc,'_',instr,'_Nflux_',dates,'.mat'];
            fck_nf = dir(nf_file);
            for imfm = 1:nmfm
                mfm = mfms{imfm};

                bl_file = [spc_base_dir,'BLdat/',spc,'_',instr,'_BL_',dates,'_',mfm,'.mat'];
                fck_bl = dir(bl_file);

                if ~isempty(fck_bl)                                    
                    process = true;
                else
                    process = false;
                end
                if process
                    PM_Lstar_file = [spc_base_dir,'Processed_Mat_Files/',spc,...
                        '_',instr,'_',dates,'_lstar','_',n_str,'_',mfm,'_',proc_vers,'.mat'];
                    fck_PM_Lstar = dir(PM_Lstar_file);
                    PM_Flux_file = [spc_base_dir,'Processed_Mat_Files/',spc,...
                        '_',instr,'_',dates,'_flux_',proc_vers,'.mat'];
                    PM_Pos_file = [spc_base_dir,'Processed_Mat_Files/',spc,...
                        '_',instr,'_',dates,'_xGEO_',proc_vers,'.mat'];
                    PM_Bsat_file = [spc_base_dir,'Processed_Mat_Files/',spc,...
                        '_',instr,'_',dates,'_real_bfield','_',proc_vers,'.mat'];

                    fprintf('%s %s %s %s %s : ',spc,instr,n_str,mfm,dates);

                    fck_PM_Flux = dir(PM_Flux_file); %check flux PMF
                            
                    if ~isempty(fck_PM_Flux) 
                        if fck_PM_Flux.datenum < fck_bl.datenum %then compute/recompute everything
                            compute = true;
                        else
                            compute = false;
                        end
                    else
                        compute = true;
                    end
                    if compute
                        fprintf('processing\n');
                        bldata = load(bl_file);

                        time = bldata.BL_time;
                        Flux = bldata.Flux;
                        xGEO = bldata.xGEO;

                        [~,~] = mkdir([spc_base_dir,'Processed_Mat_Files/']);
                        save(PM_Flux_file,'time', 'Flux','-v7.3');
                        save(PM_Pos_file,'time', 'xGEO','-v7.3');

                        % now load nf file ech data
                        load(nf_file,'ech');
                        nt = size(Flux,1);
                        ne = size(Flux,2);
                        na = size(Flux,3);

                        % now we also compute PSD here
                        PSD = nan(nt,ne,na);
                        for ie=1:ne
                            if size(ech,1) == nt
                                pc = pfunc(ech(:,ie));
                            else
                                pc = pfunc(ech(ie));
                            end
                            for ia=1:na      
                                PSD(:,ie,ia) = 1e3 ./ 2.997e10 .* Flux(:,ie,ia) ./ (pc .^ 2);
                            end
                        end
                        save([spc_base_dir,'Processed_Mat_Files/',spc,'_',instr,'_',dates,'_psd_',proc_vers,'.mat'], ...
                                'time', 'PSD','-v7.3')

                        alpha_eq_model = bldata.a_eq;
                        alpha_local = bldata.lpa;
                        Lstar = bldata.Lstar;
                        MLT = bldata.MLT;

                        alpha_eq_model(alpha_eq_model < 0 | alpha_eq_model > 90) = NaN;
                        alpha_local = degtorad(alpha_local);

                        InvK = bldata.InvI .* sqrt(bldata.Bmir .* 1e-5); % Re * G .^ 0.5

                        B_total = bldata.B_local .* 1e-5;
                        B_eq_model = bldata.B_eq .* 1e-5;
                                
                        InvMu = nan(nt,ne,na);
                        for ie=1:ne
                            if size(ech,1) == nt
                                pc = pfunc(ech(:,ie));
                            else
                                pc = pfunc(ech(ie));
                            end
                            for ia=1:na      
                                InvMu(:,ie,ia) = (pc .* sin(alpha_local(:,ia))).^2 ./ ...
                                    (2 .* mc2 .* B_total);
                            end
                        end

                        [~,~] = mkdir([spc_base_dir,'Processed_Mat_Files/']);
                        energy_channels = ech;
                        save([spc_base_dir,'Processed_Mat_Files/',spc,'_',instr,'_',dates,'_lstar','_',n_str,'_',mfm,'_',proc_vers,'.mat'], 'time', 'Lstar','-v7.3')
                        save([spc_base_dir,'Processed_Mat_Files/',spc,'_',instr,'_',dates,'_invmu_and_invk','_',n_str,'_',mfm,'_',proc_vers,'.mat'], 'time','InvMu','InvK','-v7.3')
                        save([spc_base_dir,'Processed_Mat_Files/',spc,'_',instr,'_',dates,'_alpha_and_energy','_',n_str,'_',mfm,'_',proc_vers,'.mat'], 'time', 'alpha_local',...
                        'energy_channels', 'alpha_eq_model','-v7.3');
                        save([spc_base_dir,'Processed_Mat_Files/',spc,'_',instr,'_',dates,'_bfield','_',n_str,'_',mfm,'_',proc_vers,'.mat'], 'time', 'B_total','-v7.3');
                        save([spc_base_dir,'Processed_Mat_Files/',spc,'_',instr,'_',dates,'_mlt','_',n_str,'_',mfm,'_',proc_vers,'.mat'], 'time', 'MLT', '-v7.3');
                        fprintf('done\n')
                    else
                        fprintf('ok\n')
                    end
                end
            end
        end
    end
end
