%% Written by A. C. Kellerman
%
% Correct POES data, see documentation for description on method.
%

function [D,err] = correct_poes(filename)

% Loop through data storing needed info...can add to this as desired
%D.matlabd = cell2mat(ncread(filename, 'time'));
%time = ncread(filename, 'time'); % 

year = ncread(filename, 'year');
day = ncread(filename, 'day');
msec = ncread(filename, 'msec');

year = double(year);
day = double(day);
msec = double(msec);
% begin with year
time = datenum(year,1,1); 

% add days( - 1):
time = time + (day - 1);

% add milliseconds:
time = time + msec / 1000 / 86400;
D.matlabd = time;

tsize = size(D.matlabd, 1);
%tsize
if tsize > 0
    %D.Position = reshape(cell2mat(cdfread(filename, 'Variable','geogLL')), 2,tsize);
	
	try % Since we dont' validate the variables, we load them all in try catch block
    D.alt = double(ncread(filename, 'alt'));
    D.lat = double(ncread(filename, 'lat'));
    D.lon = double(ncread(filename, 'lon'));
    
    D.L = double(ncread(filename, 'L_IGRF'));
    D.MLT = double(ncread(filename, 'MLT'));

   % Elecs0 = double(cdfread(filename, 'Variable',{'mep0E'}));
    Elecs0_1 = double(ncread(filename,'mep_ele_tel0_flux_e1'));
    Elecs0_2 = double(ncread(filename,'mep_ele_tel0_flux_e2'));
    Elecs0_3 = double(ncread(filename,'mep_ele_tel0_flux_e3'));
    Elecs0_4 = double(ncread(filename,'mep_ele_tel0_flux_e4'));

   % Elecs90 = double(cdfread(filename, 'Variable',{'mep90E'}));
    Elecs90_1 = double(ncread(filename,'mep_ele_tel90_flux_e1'));
    Elecs90_2 = double(ncread(filename,'mep_ele_tel90_flux_e2'));
    Elecs90_3 = double(ncread(filename,'mep_ele_tel90_flux_e3'));
    Elecs90_4 = double(ncread(filename,'mep_ele_tel90_flux_e4'));

   % ProtsOmni = double(cdfread(filename, 'Variable',{'mepOmni'}));
    ProtsOmni_1 = double(ncread(filename,'mep_omni_flux_p1'));
    ProtsOmni_2 = double(ncread(filename,'mep_omni_flux_p2'));
    ProtsOmni_3 = double(ncread(filename,'mep_omni_flux_p3'));

    %Prots0 = double(cdfread(filename, 'Variable',{'mep0P'}));
    Prots0_1 = double(ncread(filename,'mep_pro_tel0_flux_p1'));
    Prots0_2 = double(ncread(filename,'mep_pro_tel0_flux_p2'));
    Prots0_3 = double(ncread(filename,'mep_pro_tel0_flux_p3'));
    Prots0_4 = double(ncread(filename,'mep_pro_tel0_flux_p4'));
    Prots0_5 = double(ncread(filename,'mep_pro_tel0_flux_p5'));
    Prots0_6 = double(ncread(filename,'mep_pro_tel0_flux_p6'));

    %Prots90 = double(cdfread(filename, 'Variable',{'mep90P'}));
    Prots90_1 = double(ncread(filename,'mep_pro_tel90_flux_p1'));
    Prots90_2 = double(ncread(filename,'mep_pro_tel90_flux_p2'));
    Prots90_3 = double(ncread(filename,'mep_pro_tel90_flux_p3'));
    Prots90_4 = double(ncread(filename,'mep_pro_tel90_flux_p4'));
    Prots90_5 = double(ncread(filename,'mep_pro_tel90_flux_p5'));
    Prots90_6 = double(ncread(filename,'mep_pro_tel90_flux_p6'));
    
    %D.Elecs_pa0 = double(cdfread(filename, 'Variable',{'mep0Pitch'}));
    D.Elecs_pa0 = double(ncread(filename, 'meped_alpha_0_sat'));
    D.Elecs_pa90 = double(ncread(filename, 'meped_alpha_90_sat'));

    D.Bx = double(ncread(filename, 'Bx_sat'));
    D.By = double(ncread(filename, 'By_sat'));
    D.Bz = double(ncread(filename, 'Bz_sat'));
	catch
	warning('Some variable in nc file is missing');
	err=1;
	return	
	end

%    efsize = size(Elecs0,1);
%    pfsize = size(Prots0,1);
    nelectron_channels = 4; %efsize/tsize;
    nproton_channels = 6; %pfsize/tsize;
    
    Elecs0 = cat(2,Elecs0_1,Elecs0_2,Elecs0_3,Elecs0_4)';
    Elecs90 = cat(2,Elecs90_1,Elecs90_2,Elecs90_3,Elecs90_4)';
    ProtsOmni = cat(2,ProtsOmni_1,ProtsOmni_2,ProtsOmni_3)';
    nproton_omni_channels = 3; %size(ProtsOmni,1);

    Prots0 = cat(2,Prots0_1,Prots0_2,Prots0_3,Prots0_4,Prots0_5,Prots0_6)';
    Prots90 = cat(2,Prots90_1,Prots90_2,Prots90_3,Prots90_4,Prots90_5,Prots90_6)';
   
    D.p6omni = ProtsOmni(1,:); %in counts for contam removal
    D.p7omni = ProtsOmni(2,:); %in counts for contam removal
    D.p8omni = ProtsOmni(3,:); %in counts for contam removal
    %D.p9omni = ProtsOmni(4,:); %in counts for contam removal
    % Correct for SPE events:
    spe = find(D.p7omni > 3); %Roger et al 2013 (>36 MeV channel)
    if ~isempty(spe)
        Elecs0(:,spe) = NaN;
        Elecs90(:,spe) = NaN;
    end

    % Electron and proton channel info at: www.swpc.noaa.gov/tiger/Meped.html
    % Factor of 100 is inverse of instrument geometric factor to convert from
    % count rates to integral fluxes:
    % Electrons: Integral flux units [#/s/sr/cm^2]
    D.e30keV0deg = 100.*Elecs0(1,:);
    D.e100keV0deg = 100.*Elecs0(2,:);
    D.e300keV0deg = 100.*Elecs0(3,:);
    
    D.e30keV90deg = 100.*Elecs90(1,:);
    D.e100keV90deg = 100.*Elecs90(2,:);
    D.e300keV90deg = 100.*Elecs90(3,:);
    
    % Protons: Differential flux units [#/s/sr/cm^2/keV] for first 5 channels,
    %          Integral flux units [#/s/sr/cm^2] for 6th channel
    % From NOAA-POES Fortran reader on ViRBO: use these to convert to
    % differential flux
    ProtCal = [49.3, 147.9, 345.5, 461.8, 474.7];
    
    D.p30_80keV0deg = 100./ProtCal(1).*Prots0(1,:);
    D.p80_250keV0deg = 100./ProtCal(2).*Prots0(2,:);
    D.p250_800keV0deg = 100./ProtCal(3).*Prots0(3,:);
    D.p800_2500keV0deg = 100./ProtCal(4).*Prots0(4,:);
    D.p2500_6900keV0deg = 100./ProtCal(5).*Prots0(5,:);
    D.p6900keV0deg = 100.*Prots0(6,:);
    
    D.p30_80keV90deg = 100./ProtCal(1).*Prots90(1,:);
    D.p80_250keV90deg = 100./ProtCal(2).*Prots90(2,:);
    D.p250_800keV90deg = 100./ProtCal(3).*Prots90(3,:);
    D.p800_2500keV90deg = 100./ProtCal(4).*Prots90(4,:);
    D.p2500_6900keV90deg = 100./ProtCal(5).*Prots90(5,:);
    D.p6900keV90deg = 100.*Prots90(6,:);
   
    
    % Correct for contaminated electron channels based on Lam et al. method,
    % which is available on ViRBO in Fortran files:
    % http://virbo.org/svn/virbo/poes/noaa15+/unpack_sem2.f
    
    % Bowtie energies for proton channels
    Ebt = [52.0, 138.0, 346.0, 926.0, 2628.0];
    % Energy channel boundaries
    Echannel = [30.0, 80.0, 240.0, 800.0, 2500.0];
    % Delta E initial values
    dE1 = 0.0;
    dE2 = 0.0;

    
    % Electron correction: use exponential fits to the corrected proton data
    % and use it to correct the eletron channels
    for i = 1:length(D.matlabd)
        % Proton arrays from this time step: converted back to counts!
        ptemp0 = [D.p30_80keV0deg(i), D.p80_250keV0deg(i), D.p250_800keV0deg(i), D.p800_2500keV0deg(i), D.p2500_6900keV0deg(i)];
        ptemp90 = [D.p30_80keV90deg(i), D.p80_250keV90deg(i), D.p250_800keV90deg(i), D.p800_2500keV90deg(i), D.p2500_6900keV90deg(i)];
        
        % 0 degree data:
        % First check that there is proton data in two adjacent channels
        for ind = 2:3
            if (ptemp0(ind) > 0.0 && ptemp0(ind+1) > 0.0)
                % Calculate E0 assuming that flux = exp(E0*energy)
                E0 = (log(ptemp0(ind))-log(ptemp0(ind+1))) ./ (Ebt(ind) - Ebt(ind+1));
                E0init = 0.0;
                n = 0;
                % Now calculate a new dE based on this E0 and the bowtie
                % approximation
                while (abs((E0-E0init)/E0init) > 0.01 && n < 500)
                    E0init = E0;
                    % Calculate new dE for the first energy channel using E0
                    dE1 = ((exp(-1.0.*Ebt(ind).*E0init)) ./ E0init) .* (exp(Echannel(ind+1).*E0init) - exp(Echannel(ind).*E0init));
                    % Calculate new dE for the second energy channel using E0
                    dE2 = ((exp(-1.0.*Ebt(ind+1).*E0init)) ./ E0init) .* (exp(Echannel(ind+2).*E0init) - exp(Echannel(ind+1).*E0init));
                    % Calculate a new flux for the first energy channel
                    protemp1 = ptemp0(ind).*ProtCal(ind)./dE1;
                    % Calculate a new flux for the second energy channel
                    protemp2 = ptemp0(ind+1).*ProtCal(ind+1)./dE2;
                    % Calculate a new E0
                    E0 = (log(protemp1)-log(protemp2)) ./ (Ebt(ind) - Ebt(ind+1));
                    n = n + 1;
                end
                E0fin(ind-1) = E0;
            else
                E0fin(ind-1) = -9999.0;
            end
        end
        
        % Now calculate the appropriate corrections using E0fin
        % corr(1) = the counts from 210 to 240 keV (electrons fix 1)
        % corr(2) = the counts from 280 to 800 keV (electrons fix 2)
        % corr(3) = the counts from 440 to 800 keV (electrons fix 3)
        if E0fin(1) > -9999.0
            corr0(1) = ptemp0(2).*ProtCal(2).*(exp(E0fin(1).*Echannel(3))-exp(E0fin(1).*210))./(exp(E0fin(1).*Echannel(3))-exp(E0fin(1).*Echannel(2)))...
                + ptemp0(3).*ProtCal(3) + ptemp0(4).*ProtCal(4);
        else
            corr0(1) = ptemp0(2).*ProtCal(2).*(Echannel(3)-210)./(Echannel(3)-Echannel(2)) + ptemp0(3).*ProtCal(3) + ptemp0(4).*ProtCal(4);
        end
        
        if E0fin(2) > -9999.0
            corr0(2) = ptemp0(3).*ProtCal(3).*(exp(E0fin(2).*Echannel(4))-exp(E0fin(2).*280))./(exp(E0fin(2).*Echannel(4))-exp(E0fin(2).*Echannel(3)))...
                + ptemp0(4)*ProtCal(4);
            
            corr0(3) = ptemp0(3).*ProtCal(3).*(exp(E0fin(2).*Echannel(4))-exp(E0fin(2).*440))./(exp(E0fin(2).*Echannel(4))-exp(E0fin(2).*Echannel(3)))...
                + ptemp0(4)*ProtCal(4);
        else
            corr0(2) = ptemp0(3).*ProtCal(3).*(Echannel(4)-280)./(Echannel(4)-Echannel(3)) + ptemp0(4).*ProtCal(4);
            
            corr0(3) = ptemp0(3).*ProtCal(3).*(Echannel(4)-440)./(Echannel(4)-Echannel(3)) + ptemp0(4).*ProtCal(4);
        end
        
        % Correct appropriate channels: remember, channels are already flux units, so convert corr factors to fluxes from counts by multiplying by 100
        D.e30keV0deg(i) = D.e30keV0deg(i) - corr0(1);
        D.e100keV0deg(i) = D.e100keV0deg(i) - corr0(2);
        D.e300keV0deg(i) = D.e300keV0deg(i) - corr0(3);
        
        % 90 degree data:
        % First check that there is proton data in two adjacent channels
        for ind = 2:3
            if (ptemp90(ind) > 0.0 && ptemp90(ind+1) > 0.0)
                % Calculate E0 assuming that flux = exp(E0*energy)
                E0 = (log(ptemp90(ind))-log(ptemp90(ind+1))) ./ (Ebt(ind) - Ebt(ind+1));
                E0init = 0.0;
                n = 0;
                % Now calculate a new dE based on this E0 and the bowtie
                % approximation
                while (abs((E0-E0init)/E0init) > 0.01 && n < 500)
                    E0init = E0;
                    % Calculate new dE for the first energy channel using E0
                    dE1 = ((exp(-1.0.*Ebt(ind).*E0init)) ./ E0init) .* (exp(Echannel(ind+1).*E0init) - exp(Echannel(ind).*E0init));
                    % Calculate new dE for the second energy channel using E0
                    dE2 = ((exp(-1.0.*Ebt(ind+1).*E0init)) ./ E0init) .* (exp(Echannel(ind+2).*E0init) - exp(Echannel(ind+1).*E0init));
                    % Calculate a new flux for the first energy channel
                    protemp1 = ptemp90(ind).*ProtCal(ind)./dE1;
                    % Calculate a new flux for the second energy channel
                    protemp2 = ptemp90(ind+1).*ProtCal(ind+1)./dE2;
                    % Calculate a new E0
                    E0 = (log(protemp1)-log(protemp2)) ./ (Ebt(ind) - Ebt(ind+1));
                    n = n + 1;
                end
                E0fin(ind-1) = E0;
            else
                E0fin(ind-1) = -9999.0;
            end
        end
        
        % Now calculate the appropriate corrections using E0fin
        % corr(1) = the counts from 210 to 240 keV (electrons fix 1)
        % corr(2) = the counts from 280 to 800 keV (electrons fix 2)
        % corr(3) = the counts from 440 to 800 keV (electrons fix 3)
        if E0fin(1) > -9999.0
            corr90(1) = ptemp90(2).*ProtCal(2).*(exp(E0fin(1).*Echannel(3))-exp(E0fin(1).*210))./(exp(E0fin(1).*Echannel(3))-exp(E0fin(1).*Echannel(2)))...
                + ptemp90(3).*ProtCal(3) + ptemp90(4).*ProtCal(4);
        else
            corr90(1) = ptemp90(2).*ProtCal(2).*(Echannel(3)-210)./(Echannel(3)-Echannel(2)) + ptemp90(3).*ProtCal(3) + ptemp90(4).*ProtCal(4);
        end
        
        if E0fin(2) > -9999.0
            corr90(2) = ptemp90(3).*ProtCal(3).*(exp(E0fin(2).*Echannel(4))-exp(E0fin(2).*280))./(exp(E0fin(2).*Echannel(4))-exp(E0fin(2).*Echannel(3)))...
                + ptemp90(4)*ProtCal(4);
            
            corr90(3) = ptemp90(3).*ProtCal(3).*(exp(E0fin(2).*Echannel(4))-exp(E0fin(2).*440))./(exp(E0fin(2).*Echannel(4))-exp(E0fin(2).*Echannel(3)))...
                + ptemp90(4)*ProtCal(4);
        else
            corr90(2) = ptemp90(3).*ProtCal(3).*(Echannel(4)-280)./(Echannel(4)-Echannel(3)) + ptemp90(4).*ProtCal(4);
            
            corr90(3) = ptemp90(3).*ProtCal(3).*(Echannel(4)-440)./(Echannel(4)-Echannel(3)) + ptemp90(4).*ProtCal(4);
        end
        
        % Correct appropriate channels: remember, channels are already flux units, so convert corr factors to fluxes from counts by multiplying by 100
        D.e30keV90deg(i) = D.e30keV90deg(i) - corr90(1);
        D.e100keV90deg(i) = D.e100keV90deg(i) - corr90(2);
        D.e300keV90deg(i) = D.e300keV90deg(i) - corr90(3);
    end    
    err=0;
else
    err=1;    
end
