% POES create L* and B from MATLAB
% Author A. Kellerman

clear
addpath([getenv('DATA_ORG_DIR'),'/Functions/']);
Ini_dirs('set_numwrk',2);
global dp_dirs

data_dir = ([getenv('SAT_DATA_DIR'),'/']);

% IRBEM parameters
kext = 4; % external field model, 4 = T89
options = [1 1 4 4 0]; % controls IRBEM parameters: internal field model (IGRF), refresh time, and tracing accuracy, see doc
sysaxes = 1; %1 - GEO
mfm = 'T89';
mission='POES';

%% 1 load the data Flux, xGEO, and Nflux pitch angles. 
yr = 2016;

int = true;
int_iv = 60; %seconds 

recalc = true;

for iyr=yr:yr
    year=num2str(iyr);
    
    searchdir1=[data_dir,'POES/OriginalData/full/',year,'/'];
    sats = dir([searchdir1,'*']);
    sats = sats(3:end);
    nsats = length(sats);

    for imo=1:12
        dim=find_DIM(iyr,imo);
        for idy=1:dim
            mint=datenum(iyr, imo,idy,0,0,0);
            maxt=datenum(iyr, imo,idy,0,0,0)+1;
            strday=datestr(mint,'yyyymmdd');
            for isat=1:nsats
                sat=sats(isat).name;
                nfdir = [data_dir,mission,'/',sat,'/Nflux/'];
                bldir = [data_dir,mission,'/',sat,'/BLdat/'];
        %        pmfdir = [data_dir,mission,'/',sat,'/Processed_Mat_Files/'];
                fprintf('%s %s - ',sat,strday);

                infile = [nfdir,sat,'_SEM2_Nflux_',strday,'.mat'];
                fck1 = dir(infile);
                outfile = [bldir,sat,'_SEM2_BL_',strday,'_',mfm,'.mat'];
                fck2 = dir(outfile);

                nonf = false;
                clear Flux xGEO lpa
                if ~isempty(fck2)
                    load(outfile,'xGEO');
                    load(outfile,'lpa');
                    if ~exist('xGEO') || ~exist('lpa')
                        nonf = true;
                        fprintf('recomputing to add stuff\n')
                    end
                end

                if isempty(fck2) || fck1.datenum > fck2.datenum || recalc || nonf
                    if ~isempty(fck1)
                        fprintf('processing\n');

                        fprintf('loading nflux...');
                        data = load(infile);
                        fprintf('done\n');


                        % make ldat and bdat-type files
                        % get the solar wind for the period
                        [~,Kp] = omniwebdata(mint,maxt,38);

                        x1 = data.xGEO(:,1);
                        x2 = data.xGEO(:,2);
                        x3 = data.xGEO(:,3);

                        if int
                            time = data.flux_time;
                            lpa = data.local_pitch_angles;

                            newtime = mint:int_iv/86400:maxt;
                            newtime = newtime(1:end-1); %remove the last point
                            x1 = interp1(time,x1,newtime,'nearest');
                            x2 = interp1(time,x2,newtime,'nearest');
                            x3 = interp1(time,x3,newtime,'nearest');
                            lpa = interp1(time,data.local_pitch_angles,newtime,'nearest');
                            Flux = interp1(time,data.localflux,newtime,'nearest');
                            time = newtime;
                            clear data newtime
                        else
                            time = data.flux_time;
                            Flux = data.localflux;
                            lpa = data.local_pitch_angles;
                        end

                        xGEO = cat(2,x1,x2,x3);

                        if nonf % then just save the Flux variable
                            save(outfile,'xGEO','lpa','-append');
                        else

                            % create maginput array:
                            nt = length(time);

                            maginput = nan(nt,25); % see doco for other array elements
                            %maginput(:,1) = interp1(Kp.time,Kp.arr/10,time,'nearest'); %interpolate using nearest
                            maginput(:,1) = interp1(Kp.time,Kp.arr,time,'nearest'); %interpolate using nearest

                            max_size = 10000; %max size parallized / cpu (sent to fortran)

                            na = size(lpa,2);

                            % pre-allocate our B-field arrays
                            B_local = nan(nt,1);
                            B_eq = nan(nt,1);
                            MLT = nan(nt,1);
                            Lstar = nan(nt,na);
                            Lm = nan(nt,na);
                            InvI = nan(nt,na);
                            Bmir = nan(nt,na);
                            a_eq = nan(nt,na);

                            % break it into parts
                            sinx = 1;
                            einx = min([max_size nt]) ;

                            while sinx <= nt
                                nt2 = einx - sinx + 1;

                                ttmp = time(sinx:einx);
                                x1tmp = x1(sinx:einx);
                                x2tmp = x2(sinx:einx);
                                x3tmp = x3(sinx:einx);
                                maginput_tmp = maginput(sinx:einx,:);
                                patmp = lpa(sinx:einx,:);

                                Bl = nan(nt2,1);
                                Beq = nan(nt2,1);
                                mlt = nan(nt2,1);
                                L = nan(nt2,na);
                                L2 = nan(nt2,na);
                                aeq = nan(nt2,na);
                                Bm = nan(nt2,na);
                                I = nan(nt2,na);

                                tic
                                % use parallal processing
                                fprintf('computing B, L*\n');
                                %parfor it=1:nt2                                
                                for it=1:nt2                                
                                    if mod(it,10) == 0
                                        fprintf('B,L* (%s %s) - total: %i, %i-%i: %i\n',sat,strday,nt,sinx,einx,it);
                                    end
                                    if ~isnan(x1tmp(it)) && ~isnan(x2tmp(it)) && ~isnan(x3tmp(it))
                                        [~,B1] = onera_desp_lib_get_field(kext,options,sysaxes,ttmp(it),x1tmp(it),...
                                            x2tmp(it),x3tmp(it),maginput_tmp(it));
                                        Bl(it) = B1;

                                        for ia=1:na
                                            alpha = patmp(it,ia);
                                            [A2,B2,C2,D2,E2,F2] = onera_desp_lib_make_lstar_shell_splitting(...
                                                kext,options,sysaxes,ttmp(it),x1tmp(it),x2tmp(it),x3tmp(it),alpha,maginput_tmp(it,:));

                                            L2(it,ia) = A2;
                                            L(it,ia) = B2;
                                            Bm(it,ia) = C2;
                                            Beq(it) = D2;
                                            I(it,ia) = E2;
                                            mlt(it) = F2;
                                            aeq(it,ia) = asin(sqrt(D2./B1 .* sin(alpha.*pi/180).^2)); 
                                        end
                                    end
                                end

                                telap = toc;
                                persec =  telap / nt2;
                                
                                B_local(sinx:einx) = Bl;
                                B_eq(sinx:einx) = Beq;
                                MLT(sinx:einx) = mlt;
                                Lstar(sinx:einx,:) = L;
                                Lm(sinx:einx,:) = L2;
                                a_eq(sinx:einx,:) = aeq;
                                InvI(sinx:einx,:) = I;
                                Bmir(sinx:einx,:) = Bm;

                                sinx = einx + 1;
                                einx = sinx + max_size - 1;
                                einx = min([einx nt]) ;
                            end
                            BL_time = time;
                            [~,~] = mkdir(bldir);
                            save(outfile,'BL_time','B_local','B_eq','MLT','Lstar','Lm','a_eq',...
                                'Flux','lpa','InvI','Bmir','xGEO','telap','persec');
                        end
                    end
                else
                    fprintf('ok\n');
                end
            end
        end
    end
end
