% POES create nflux files
% Author A. Kellerman

clear

addpath([getenv('DATA_ORG_DIR'),'/Functions/']);
Ini_dirs
global dp_dirs

data_dir = ([getenv('SAT_DATA_DIR'),'/']);

close all
ech_base = [30 100 300 1000]

Ini_dirs
mission='POES'

for iyr=2016:2016
    year=num2str(iyr);
    
    searchdir1=[data_dir,'POES/OriginalData/full/',year,'/'];
    sats = dir([searchdir1,'*']);
    sats = sats(3:end);
    nsats = length(sats);
 
    for imo=3:3
        dim=find_DIM(iyr,imo);
        for idy=1:dim
            mint=datenum(iyr, imo,idy,0,0,0);
            maxt=datenum(iyr, imo,idy,0,0,0)+1;
            strday=datestr(mint,'yyyymmdd');

            for isat=1:nsats
                sat=sats(isat).name;
                newdir = [data_dir,mission,'/',sat,'/Nflux/'];
                outfile = [newdir,sat,'_SEM2_Nflux_',strday,'.mat'];
                nf_fck = dir(outfile);

                fprintf('%s %s...',sat,strday)

                searchdir2=[searchdir1,sat,'/'];
                spc_fck = [searchdir2,'*',strday,'_proc.mat'];
                pc_fck=dir(spc_fck);

                nfiles=length(pc_fck);
                if nfiles == 1

                    process = false;
                    if ~isempty(nf_fck)
                        if pc_fck.datenum > nf_fck.datenum
                            % raw file is newer, so reprocess
                            process = true;
                        end
                    else
                        process = true; %if there is no file, then process
                    end

                    if ~process
                        fprintf('file exists and ok\n');
                    else
                        fprintf('processing ....');
                        
                        [~,~] = system(['mkdir -p ',newdir]);
                        e_90_30 = [];
                        e_90_100 = [];
                        e_90_300 = [];
                        e_0_30 = [];
                        e_0_100 = [];
                        e_0_300 = [];
                        p_90_2500_6900 = [];
                        p_0_2500_6900 = [];
                        p_90_6900 = [];
                        p_0_6900 = [];
                        p7_omni_cnts = [];

                        Flux = [];
                        e_matday = [];
                        energy = [];
                        McIlwain_L = [];
                        integralflux = [];
                        
                        load([searchdir2,pc_fck.name]);
                        
                        nt = length(e_matday);
                        energy=[30 100 300 450 700 850 1000];
                        ne = length(energy);
                        localflux = nan(nt,ne,2);
                        local_pitch_angles = nan(nt,2);
                        ech = energy;
                        
                        noise_floor = 10.^2.3;                      
                        
                        e_90_30(e_90_30 < noise_floor) = NaN;
                        e_90_100(e_90_100 < noise_floor) = NaN;
                        e_90_300(e_90_300 < noise_floor) = NaN;
                        
                        e_0_30(e_0_30 < noise_floor) = NaN;
                        e_0_100(e_0_100 < noise_floor) = NaN;
                        e_0_300(e_0_300 < noise_floor) = NaN;
                        
                        p_0_2500_6900(p_0_2500_6900 < noise_floor) = NaN;
                        p_90_2500_6900(p_90_2500_6900 < noise_floor) = NaN;
                        
                        p_0_6900(p_0_6900 < noise_floor) = NaN;
                        p_90_6900(p_90_6900 < noise_floor) = NaN;
                        
                        catflux_90 = cat(2,e_90_30,e_90_100,e_90_300,p_90_6900);
                        catflux_0 = cat(2,e_0_30,e_0_100,e_0_300,p_0_6900);
                        
                        protoninx_90 = find(~isnan(p_90_2500_6900));
                        protoninx_0 = find(~isnan(p_0_2500_6900));
                        
                        catflux_90(protoninx_90,:) = NaN;
                        catflux_0(protoninx_0,:) = NaN;
                        
                        integralflux(:,:,1) = catflux_0;
                        integralflux(:,:,2) = catflux_90;
                        intech=[0.03 0.1 0.3 1];
                        tintchannels = length(intech);
                        p7_omni_cnts = reshape(p7_omni,[],1);
                
                        flux_time=reshape(e_matday,[],1);
                        %parfor it=1:nt
                        for it=1:nt
                            L = McIlwain_L(it);
                            %str=['L = ',num2str(McIlwain_L(it))];
                            %fprintf(str);
                            % if it == 1 || rem(it,1000) == 0
                            %     fprintf('%s\n',[num2str(it),'/',num2str(nt)]);
                            % end
                            
                            if L <= 15
                                maxf = max(catflux_90(it,:));
                                minf = min(catflux_90(it,:));
                                %                             subplot(3,1,1)
                                %                             plot(ech_base,catflux_90(it,:),'.')
                                fsize=12;
                                df = maxf-minf;
                                
                                %J = Ae^BE
                                %J = Ae^BE / B
                                %log(J) = log(A)-BE
                                %log(J) = log(A)-BE-log(B)
                                
                                %J = e^A E^B
                                % log(J) = A+B log(E)
                                %% convert to differential
                                f_90 = [];
                                x_90 = [];
                                for fi=1:3
                                    Jint1 = catflux_90(it,fi);
                                    Jint2 = catflux_90(it,fi+1);
                                    E1 = ech_base(fi);
                                    E2 = ech_base(fi+1);
                                    B = (log(Jint2)-log(Jint1))/(E1 - E2);
                                    A = (B.*Jint1)/exp(-B.*E1);
                                    charJ = A .* (exp(-B.*E1)./B - exp(-B.*E2)./B)./(E2-E1); %using analytical solution -Ae^-Bx/B | 2 and 1
                                    charE = - log(charJ/A) ./ B;
                                    x_90(fi) = charE;
                                    f_90(fi) = A.*exp(-B.*charE);
                                end
                                y_90 = log(f_90);
                                
                                f_0 = [];
                                x_0 = [];
                                for fi=1:3
                                    Jint1 = catflux_0(it,fi);
                                    Jint2 = catflux_0(it,fi+1);
                                    E1 = ech_base(fi);
                                    E2 = ech_base(fi+1);
                                    B = (log(Jint2)-log(Jint1))/(E1 - E2);
                                    A = (B.*Jint1)/exp(-B.*E1);
                                    charJ = A .* (exp(-B.*E1)./B - exp(-B.*E2)./B)./(E2-E1); %using analytical solution -Ae^-Bx/B | 2 and 1
                                    charE = - log(charJ/A) ./ B;
                                    x_0(fi) = charE;
                                    f_0(fi) = A.*exp(-B.*charE);
                                end
                                y_0 = log(f_0);
                                m_90 = [];
                                m_0 = [];
                                
                                cnt=1;
                                for ie=1:length(y_90)-1
                                    for i2=ie+1:length(y_90)
                                        y1=y_90(ie);
                                        y2=y_90(i2);
                                        x1=x_90(ie);
                                        x2=x_90(i2);
                                        m_90(cnt) = (y2-y1)/(x2-x1);
                                        
                                        y1=y_0(ie);
                                        y2=y_0(i2);
                                        x1=x_0(ie);
                                        x2=x_0(i2);
                                        m_0(cnt) = (y2-y1)/(x2-x1);
                                        
                                        cnt=cnt+1;
                                    end
                                end
                                mean_m = nanmean(m_90);
                                B = mean_m; %b = -m;
                                c = y_90(1)-B.*x_90(1);
                                A = exp(c);
                                newy_90 = A.*exp(B.*energy);
                                %                             subplot(3,1,3)
                                %                             plot(energy,newy_90,'--g')
                                %                             hold on
                                %                             plot(energy,newy_90,'.k')
                                %                             hold off
                                %
                                mean_m = nanmean(m_0);
                                B = mean_m; %b = -m;
                                
                                c = y_0(1)-B.*x_0(1);
                                A = exp(c);
                                newy_0 = A.*exp(B.*energy);
                                %                             subplot(3,1,3)
                                %                             plot(energy,newy_0,'--g')
                                %                             hold on
                                %                             plot(energy,newy_0,'.k')
                                %                             hold off
                                %                                 if ~isempty(find(~isnan(newy_0),1))
                                %                                     drawnow
                                %                                 end
                                tflux = zeros(length(energy),2);
                                tpa = zeros(1,2);
                                tpa(1) = e_0_pa(it);
                                tpa(2) = e_90_pa(it);
                                
                                local_pitch_angles(it,:) = tpa;

                                if size(energy,1) > 1
                                    energy = energy';
                                end
                                
                                tflux(:,1) = newy_0;
                                tflux(:,2) = newy_90;
                                localflux(it,:,:) = tflux;
                            end
                        end
                        for ipa=1:2
                            lpa = local_pitch_angles(:,ipa);
                            onpa = find(lpa > 90);
                            if ~isempty(onpa) % correct for > 90
                                lpa(onpa) = 180 - lpa(onpa);
                                local_pitch_angles(:,ipa) = lpa; 
                            end
                        end

                        xGEO = onera_desp_lib_rotate([alt, geolat, geolon],'gdz2geo');
                        
                        %save(outfile,'flux_time','intech','ech','local_pitch_angles','localflux','integralflux','tchannels','tintchannels','p7_omni_cnts','xGEO');

                        okt = find(flux_time >= mint & flux_time < maxt);
                        if ~isempty(okt)
                            flux_time = flux_time(okt);
                            xGEO = xGEO(okt,:);
                            local_pitch_angles = local_pitch_angles(okt,:);
                            localflux = localflux(okt,:,:);
                            integralflux = integralflux(okt,:,:);
                            p7_omni_cnts = p7_omni_cnts(okt);
                            fprintf('done\n');
                            MeV = 0;
                            ech = ech ./ 1000;
                            save(outfile,'flux_time','intech','ech','local_pitch_angles','localflux','integralflux',...
                                'p7_omni_cnts','xGEO','MeV','-v7.3');
                        else
                            fprintf('no data in range\n')
                        end
                    end
                end
            end
        end
    end
end
