% POES load from netcdf files
% Author A. Kellerman

clear

if ~isdeployed
    addpath([getenv('DATA_ORG_DIR'),'/Functions/']);
end

Ini_dirs
global dp_dirs

data_dir = ([getenv('SAT_DATA_DIR'),'/']);

recalc = false;

degs = {'0', '90'};
energy = {'30', '100', '300'};
penergy={'30_80','80_250', '250_800', '800_2500', '2500_6900', '6900'};

for iyr=[2013, 2017]
%for iyr=[2013]
    year=num2str(iyr);
    
    searchdir1=[data_dir, 'POES/OriginalData/full/',year,'/']
    sats = dir([searchdir1,'*']);
    sats = sats(3:end); 
    nsats = length(sats);
    for imo=1:12 % all months to process
    %for imo=1 % one month to process
        dim=find_DIM(iyr,imo);
        for idy=1:dim
            mint=datenum(iyr, imo,idy,0,0,0);
            maxt=datenum(iyr, imo,idy,0,0,0)+1;
            strday=datestr(mint,'yyyymmdd');
            %for isat=1:nsats
            for isat=1:numel(sats) % All one sattelite
            %for isat=4 % one sattelite
                searchdir2=[data_dir,'POES/OriginalData/full/',year,'/',sats(isat).name,'/'];
                sfiles = [searchdir2,'*',strday,'_proc.nc'];
                files = dir(sfiles);
                nfiles = length(files);
                fprintf('%s, %s -',strday,sats(isat).name);
                if nfiles == 0
                    fprintf('no file\n')
                end
                for ifile=1:nfiles
                    infile=files(ifile).name;

                    outfile = regexprep(infile, '.nc','.mat');
                    
                    filechk = dir([searchdir2,outfile]);
                    process=1;
                    if ~isempty(filechk)
                        ofdate = datenum(filechk.date);
                        cdfdate = datenum(files.date);
                        if cdfdate > ofdate
                            process=1;
                            fprintf('CDF file is newer than mat file:\n');
                            fprintf('CDF date: %s\n',files.date);
                            fprintf('MAT date: %s\n',filechk.date);
                            fprintf('Processing %s...',sats(isat).name)
                        else
                            fprintf('file exists, up to date\n');
                            process=0;
                        end
                    else
                        fprintf('No MAT file, processing %s...',sats(isat).name)
                    end
                    if process || recalc
                        if ~process && recalc
                            fprintf('Recalculating %s...',sats(isat).name)
                        end
                        clear e_matday
                        clear McIlwain_L
                        clear Bscalar
                        
                        for degi=1:2
                            for ei=1:3
                                eval(['e_',degs{degi},'_',energy{ei},' = [];']);
                            end
                        end
                        for degi=1:2
                            for pi=1:6
                                eval(['p_',degs{degi},'_',penergy{pi},' = [];']);
                            end
                        end

                        e_matday = [];
                        p7_omni = []; 
                        McIlwain_L = [];
                        geolat = [];
                        geolon = [];
                        alt = [];
                        Bscalar = [];
                        file=[searchdir2,infile];
                        
                        %[poes_data,err] = ReadPOESFiles_ngdc(file);
                        [poes_data,err] = correct_poes(file);

                        if err == 0
                            %there's a problem with n18 so do the following to
                            %remove all issues (dates in June for Jan)
                            zz=find(poes_data.matlabd >= mint & poes_data.matlabd < maxt);
                            
                            if ~isempty(zz)
                                e_matday = poes_data.matlabd(zz);
                                
                                for degi=1:2
                                    for ei=1:3
                                        eval(['e_',degs{degi},'_',energy{ei},' = cat(1, e_',degs{degi},...
                                            '_',energy{ei},', poes_data.e',energy{ei},'keV',degs{degi},'deg(zz)'');']);
                                    end
                                end
                                for degi=1:2
                                    for pi=1:6
                                        eval(['p_',degs{degi},'_',penergy{pi},' = cat(1, p_',degs{degi},...
                                            '_',penergy{pi},', poes_data.p',penergy{pi},'keV',degs{degi},'deg(zz)'');']);
                                    end
                                end
                                p7_omni = cat(1,p7_omni,poes_data.p7omni); % in counts
                                
                                z=find(poes_data.L < 0);
                                poes_data.L(z) = NaN;
                                
                                McIlwain_L = cat(1, McIlwain_L, poes_data.L(zz));
                                geolat = cat(1,geolat, poes_data.lat(zz));
                                geolon = cat(1,geolon, poes_data.lon(zz));
                                alt = cat(1, alt, poes_data.alt(zz));
                                tmpsz = size(e_matday);
                               % fprintf('%s\n', ['total array size: ',num2str(tmpsz(1)),'.']);                                
                               % fprintf('%s\n', [datestr(min(e_matday)), ' to ',datestr(max(e_matday))]);
                            end
                            
                            [tmp, inx]=sort(e_matday);
                            e_matday = e_matday(inx)';
                            McIlwain_L = McIlwain_L(inx)';
                            geolat = geolat(inx);
                            geolon = geolon(inx);
                            alt = alt(inx);
                            p7_omni = p7_omni(inx);
                            
                            for degi=1:2
                                for ei=1:3
                                    eval(['e_',degs{degi},'_',energy{ei},' = e_',degs{degi},'_',energy{ei},'(inx);'])
                                end
                                for pi=1:6
                                    eval(['p_',degs{degi},'_',penergy{pi},' = p_',degs{degi},'_',penergy{pi},'(inx);'])
                                end
                            end
                            
                            e_0_pa = poes_data.Elecs_pa0;
                            e_90_pa = poes_data.Elecs_pa90;
                            
                            save([searchdir2,outfile], 'e_0_30', 'e_0_100', 'e_0_300', 'e_90_30',...
                                'e_90_100', 'e_90_300', 'e_matday', 'McIlwain_L', '-v7.3',...
                                'p_0_30_80','p_0_80_250', 'p_0_250_800', 'p_0_800_2500', 'p_0_2500_6900', 'p_0_6900',...
                                'p_90_30_80','p_90_80_250', 'p_90_250_800', 'p_90_800_2500', 'p_90_2500_6900', ...
                                'p_90_6900','geolat','geolon','alt','e_0_pa','e_90_pa','p7_omni');
                            fprintf('done\n');
                        else
                            fprintf('error\n');
                        end
                    end
                end
            end
        end
    end
end
