% POES create nflux files
% Author A. Kellerman

clear
tic

if ~isdeployed
    addpath([getenv('DATA_ORG_DIR'),'/Functions/']);
end

data_dir = ([getenv('SAT_DATA_DIR'),'/']);

% ncpu=16; % standart
ncpu=64; % rbm7 (max is 128), 64 seems faster
%ncpu=48; % rbm7 (max is 128), 64 seems faster
%ncpu=32; % rbm7 (max is 128), 64 seems faster
%ncpu=16; % rbm7 (max is 128), 64 seems faster

parpool(ncpu)
toc
tic

Ini_dirs('set_numwrk',ncpu)

mission='POES'
start_year = 2013;
end_year = 2017;
%end_year = 2013;


pp_flag = true;

% don't change, unless new energies are defined
ech_base = [30 100 300 1000]

% specify years here
for iyr=start_year:end_year
    year=num2str(iyr);
    
    searchdir1=[data_dir,'POES/OriginalData/full/',year,'/'];
    sats = dir([searchdir1,'*']);
    sats = sats(3:end);
    nsats = length(sats);
    for isat=1:nsats
        sat=sats(isat).name;
        newdir = [data_dir,mission,'/',sat,'/Nflux/'];

        for imo=1:12
        %for imo=1:1 % testing

            dates = make_dates(iyr,imo);
            fprintf('%s %s...',sat,dates)

            outfile = [newdir,sat,'_SEM2_Nflux_',dates,'.mat'];
            fckmo = dir(outfile);

            process = false;
            if isempty(fckmo)
                process = true;
            end

            if ~process
                fprintf('file exists and ok\n');
            else
                fprintf('processing\n');

                ft_a = [];
                xg_a = [];
                lpa_a = [];
                lfx_a = [];
                ifx_a = [];
                p7_a = [];

                dim=find_DIM(iyr,imo);
                
                for idy=1:dim
                    mint=datenum(iyr, imo,idy,0,0,0);
                    maxt=datenum(iyr, imo,idy,0,0,0)+1;
                    strday=datestr(mint,'yyyymmdd');

                    searchdir2=[searchdir1,sat,'/'];
                    sfile = [searchdir2,'*',strday,'_proc.mat'];
                    file=dir(sfile);

                    nfiles=length(file);
                    if nfiles == 1
                       
                        if pp_flag
                            try
                                parpool(ncpu)
                            catch end
                            pp_flag = false;
                        end
                        system(['mkdir -p ',newdir]);
                        fprintf('%s %s processing...',sat, strday);
                        e_90_30 = [];
                        e_90_100 = [];
                        e_90_300 = [];
                        e_0_30 = [];
                        e_0_100 = [];
                        e_0_300 = [];
                        p_90_2500_6900 = [];
                        p_0_2500_6900 = [];
                        p_90_6900 = [];
                        p_0_6900 = [];
                        p7_omni_cnts = [];

                        ech = [];
                        Flux = [];
                        e_matday = [];
                        McIlwain_L = [];
                        integralflux = [];
                        
                        load([searchdir2,file(1).name]);
                        
                        nt = length(e_matday);
                        energy=[30 100 300 450 700 850 1000];
                        if size(energy,1) > 1
                            energy = energy';
                        end
                        ne = length(energy);
                        localflux = nan(nt,ne,2);
                        local_pitch_angles = nan(nt,2);
                        ech = energy;
                        
                        noise_floor = 10.^2.3;                      
                        
                        e_90_30(e_90_30 < noise_floor) = NaN;
                        e_90_100(e_90_100 < noise_floor) = NaN;
                        e_90_300(e_90_300 < noise_floor) = NaN;
                        
                        e_0_30(e_0_30 < noise_floor) = NaN;
                        e_0_100(e_0_100 < noise_floor) = NaN;
                        e_0_300(e_0_300 < noise_floor) = NaN;
                        
                        p_0_2500_6900(p_0_2500_6900 < noise_floor) = NaN;
                        p_90_2500_6900(p_90_2500_6900 < noise_floor) = NaN;
                        
                        p_0_6900(p_0_6900 < noise_floor) = NaN;
                        p_90_6900(p_90_6900 < noise_floor) = NaN;
                        
                        catflux_90 = cat(2,e_90_30,e_90_100,e_90_300,p_90_6900);
                        catflux_0 = cat(2,e_0_30,e_0_100,e_0_300,p_0_6900);
                        
                        protoninx_90 = find(~isnan(p_90_2500_6900));
                        protoninx_0 = find(~isnan(p_0_2500_6900));
                        
                        catflux_90(protoninx_90,:) = NaN;
                        catflux_0(protoninx_0,:) = NaN;
                        
                        integralflux(:,:,1) = catflux_0;
                        integralflux(:,:,2) = catflux_90;
                        intech=[0.03 0.1 0.3 1];
                        tintchannels = length(intech);
                        p7_omni_cnts = reshape(p7_omni,[],1);
                
                        flux_time=reshape(e_matday,[],1);

                        % computations in parallel
                        parfor it=1:nt
                            L = McIlwain_L(it);
                            
                            if L <= 15
                                maxf = max(catflux_90(it,:));
                                minf = min(catflux_90(it,:));
                                %                             subplot(3,1,1)
                                %                             plot(ech_base,catflux_90(it,:),'.')
                                fsize=12;
                                df = maxf-minf;
                                
                                %% convert to differential
                                f_90 = [];
                                x_90 = [];
                                for fi=1:3
                                    Jint1 = catflux_90(it,fi);
                                    Jint2 = catflux_90(it,fi+1);
                                    E1 = ech_base(fi);
                                    E2 = ech_base(fi+1);
                                    if Jint2 > 0 && Jint1 > 0
                                        B = (log(Jint2)-log(Jint1))/(E1 - E2);
                                        A = (B.*Jint1)/exp(-B.*E1);
                                        charJ = A .* (exp(-B.*E1)./B - exp(-B.*E2)./B)./(E2-E1); %using analytical solution -Ae^-Bx/B | 2 and 1
                                        charE = - log(charJ/A) ./ B;
                                        x_90(fi) = charE;
                                        f_90(fi) = A.*exp(-B.*charE);
                                    else
                                        x_90(fi) = NaN;
                                        f_90(fi) = NaN;
                                    end
                                end
                                f_90(f_90 <= 0) = NaN;
                                y_90 = log(f_90);
                                
                                f_0 = [];
                                x_0 = [];
                                for fi=1:3
                                    Jint1 = catflux_0(it,fi);
                                    Jint2 = catflux_0(it,fi+1);
                                    E1 = ech_base(fi);
                                    E2 = ech_base(fi+1);
                                    if Jint2 > 0 && Jint1 > 0
                                        B = (log(Jint2)-log(Jint1))/(E1 - E2);
                                        A = (B.*Jint1)/exp(-B.*E1);
                                        charJ = A .* (exp(-B.*E1)./B - exp(-B.*E2)./B)./(E2-E1); %using analytical solution -Ae^-Bx/B | 2 and 1
                                        charE = - log(charJ/A) ./ B;
                                        x_0(fi) = charE;
                                        f_0(fi) = A.*exp(-B.*charE);
                                    else
                                        x_0(fi) = NaN;
                                        f_0(fi) = NaN;
                                    end
                                end
                                f_0(f_0 <= 0) = NaN;

                                y_0 = log(f_0);
                                m_90 = [];
                                m_0 = [];
                                
                                cnt=1;
                                for ie=1:length(y_90)-1
                                    for i2=ie+1:length(y_90)
                                        y1=y_90(ie);
                                        y2=y_90(i2);
                                        x1=x_90(ie);
                                        x2=x_90(i2);
                                        m_90(cnt) = (y2-y1)/(x2-x1);
                                        
                                        y1=y_0(ie);
                                        y2=y_0(i2);
                                        x1=x_0(ie);
                                        x2=x_0(i2);
                                        m_0(cnt) = (y2-y1)/(x2-x1);
                                        
                                        cnt=cnt+1;
                                    end
                                end
                                mean_m = nanmean(m_90);
                                B = mean_m; %b = -m;
                                c = y_90(1)-B.*x_90(1);
                                A = exp(c);
                                newy_90 = A.*exp(B.*energy);
                                %
                                mean_m = nanmean(m_0);
                                B = mean_m; %b = -m;
                                
                                c = y_0(1)-B.*x_0(1);
                                A = exp(c);
                                newy_0 = A.*exp(B.*energy);
                                tflux = zeros(length(energy),2);
                                tpa = zeros(1,2);
                                tpa(1) = e_0_pa(it);
                                tpa(2) = e_90_pa(it);
                                
                                local_pitch_angles(it,:) = tpa;

                                
                                tflux(:,1) = newy_0;
                                tflux(:,2) = newy_90;
                                localflux(it,:,:) = tflux;
                            end
                        end
                        for ipa=1:2
                            lpa = local_pitch_angles(:,ipa);
                            onpa = find(lpa > 90);
                            if ~isempty(onpa) % correct for > 90
                                lpa(onpa) = 180 - lpa(onpa);
                                local_pitch_angles(:,ipa) = lpa; 
                            end
                        end

                        xGEO = onera_desp_lib_rotate([alt, geolat, geolon],'gdz2geo');
                        
                        %save(outfile,'flux_time','intech','ech','local_pitch_angles','localflux','integralflux','tchannels','tintchannels','p7_omni_cnts','xGEO');

                        okt = find(flux_time >= mint & flux_time < maxt);
                        if ~isempty(okt)
                            ft_a = cat(1,ft_a,flux_time(okt));
                            xg_a = cat(1,xg_a,xGEO);
                            lpa_a = cat(1,lpa_a,local_pitch_angles(okt,:));
                            lfx_a = cat(1,lfx_a,localflux(okt,:,:));
                            ifx_a = cat(1,ifx_a,integralflux(okt,:,:));
                            p7_a = cat(1,p7_a,p7_omni_cnts(okt));
                            fprintf('ok\n');
                        else
                            fprintf('no data in range\n')
                        end
                    end
                end
                if ~isempty(ft_a)
                    flux_time = ft_a;
                    local_pitch_angles = lpa_a;
                    localflux = lfx_a;
                    integralflux = ifx_a;
                    p7_omni_cnts = p7_a;
                    xGEO = xg_a;
                    ech = ech ./ 1000; % to MeV
                    MeV = 0; % flat to let the code know flux units are #/keV
                    save(outfile,'flux_time','intech','ech','local_pitch_angles','localflux','integralflux',...
                        'p7_omni_cnts','xGEO','MeV','-v7.3');
                end
            end
            toc
        end
    end
end
